<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

/**
 * XtdirControllerObjects
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class XtdirControllerObjects extends FOFController
{
	/**
	 * Saves the order of the items
	 *
	 * @return  void
	 */
	public function saveorder()
	{
		$controller = FOFController::getTmpInstance($this->config['option'], 'promoentries', $this->config);
		$status = $controller->execute('saveorderfromentry');

		if ($status)
		{
			// Redirect

			if ($customURL = $this->input->get('returnurl', '', 'string'))
			{
				$customURL = base64_decode($customURL);
			}

			$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);
			$this->setRedirect($url);
		}
	}

	/**
	 * entryToggleEnabled
	 *
	 * @return	void
	 */
	public function entryToggleEnabled()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$entry_id = $this->input->get('entryId', null, 'int');

		if ($entry_id)
		{
			SobiproHelper::entryToggleEnabled($entry_id);
		}
	}

	/**
	 * entryToggleApproval
	 *
	 * @return	void
	 */
	public function entryToggleApproval()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$entry_id = $this->input->get('entryId', null, 'int');

		if ($entry_id)
		{
			SobiproHelper::entryToggleApproval($entry_id);
		}
	}

	/**
	 * entryTogglePromoted
	 *
	 * @return	void
	 */
	public function entryTogglePromoted()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$message = array(
						'status' => false,
						'error_message' => JText::_('COM_XTDIR_VIEW_PROMOTION_FAILED_ERR')
		);

		$entry_id = $this->input->get('entryId', null, 'int');

		if ($entry_id)
		{
			PromotedHelper::entryTogglePromoted($entry_id);

			$finalpromotedentries = FOFModel::getTmpInstance('FinalSectionPromoted', 'XtdirModel');
			$finalpromotedentries->setState('entry_id', $entry_id);
			$finalpromotedentry = $finalpromotedentries->getFirstItem(true);

			if (isset($finalpromotedentry->id))
			{
				$message = array(
								'status' => true,
								'error_message' => 'Ok',
								'finalpromotedentry' => array(
												'promoentry_id' => $finalpromotedentry->promoentry_id,
												'entry_id' => $finalpromotedentry->entry_id,
												'promotype_id' => $finalpromotedentry->promotype_id,
												'ordering' => $finalpromotedentry->ordering
								)
				);
			}
		}

		$message = json_encode($message);
		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * index.
	 *
	 * @return	void
	 */
	public function index()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		TreeIndexer::index(true);

		// Redirect
		if ($customURL = $this->input->get('returnurl', '', 'string'))
		{
			$customURL = base64_decode($customURL);
		}

		// ' . FOFInflector::pluralize($this->view)
		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=objects';
		$this->setRedirect($url);
	}

	/**
	 * batch.
	 *
	 * @return	void
	 */
	public function batch()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		// Get the ID or list of IDs from the request or the configuration
		$cid = $this->input->get('cid', array(), 'array');

		if (!empty($cid))
		{
			foreach ($cid as $entry_id)
			{
				PromotedHelper::entryTogglePromoted($entry_id);
			}
		}

		$status = true;

		// Check if i'm using an AJAX call, in this case there is no need to redirect
		$format = $this->input->get('format', '', 'string');

		if ($format == 'json')
		{
			echo json_encode($status);

			return;
		}

		// Redirect
		if ($customUrl = $this->input->get('returnurl', '', 'string'))
		{
			$customUrl = base64_decode($customUrl);
		}

		$url = !empty($customUrl) ? $customUrl : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);

		if (!$status)
		{
			$this->setRedirect($url, $model->getError(), 'error');
		}
		else
		{
			$this->setRedirect($url);
		}
	}
}
