<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once JPATH_SITE . '/components/com_content/helpers/route.php';

/**
 * CategoriesHelper Helper class.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
 */
class CategoriesHelper
{
	/**
	 * getListNLevel
	 *
	 * @param   mixed  $section_id     Params
	 * @param   int    $start_level    Params
	 * @param   int    $end_level      Params
	 * @param   mixed  $count          Params
	 * @param   mixed  $query_order    Params
	 * @param   mixed  $require_stats  Params
	 *
	 * @return  array
	 */
	public static function getListNLevel($section_id, $start_level, $end_level, $count, $query_order, $require_stats)
	{
		$logger = XtdirLogger::getInstance();
		$logger->log(
				JLog::INFO, "getListNLevel: section_id: {$section_id},
				categorystartlevel: {$start_level},
				categorymode: {$end_level},
				count: {$count},
				sorder: {$query_order},
				require_stats: {$require_stats}"
		);

		$results = array();

		$parents = array();
		$parents[] = $section_id;

		for ($level = 1; $level <= $end_level; $level++)
		{
			$next_parents = array();
			$n = count($parents);

			for ($i = 0; ($i < $n); $i++)
			{
				$parent_id = $parents[$i];
				$rows = self::_getCategoriesFromParent($parent_id, $require_stats, $query_order, $count);
				$results[$parent_id] = $rows;
				$next_parents = array_merge($next_parents, array_keys($rows));
			}

			$parents = $next_parents;
		}

		return $results;
	}

	/**
	 * _getCategories
	 *
	 * @param   int    $parent_id      Params
	 * @param   mixed  $require_stats  Params
	 * @param   mixed  $query_order    Params
	 * @param   mixed  $count          Params
	 * @param   mixed  $sid_list       Params
	 *
	 * @return  array
	 */
	private static function _getCategoriesFromParent($parent_id, $require_stats, $query_order, $count, $sid_list = null)
	{
		$db = JFactory::getDBO();
		$qorder = self::getQueryOrder($query_order, $require_stats);
		$rows = null;

		if ((is_array($parent_id)) && (count($parent_id) > 0))
		{
			$qparent = join(',', $parent_id);
			$qparent = ' cc.pid IN (' . $qparent . ')';
		}
		else
		{
			$parent_id = (int) $parent_id;

			if ($parent_id)
			{
				$qparent = ' cc.pid = ' . $db->Quote($parent_id);
			}
			else
			{
				// No enough levels, invalidating
				$qparent = ' 1 = 2';
			}
		}

		$isSobiPro11 = SobiproHelper::isSobiPro11();

		$query = 'SELECT c.id AS id, c.name AS name, c.nid AS alias, parent as cid, ' .
			($isSobiPro11 ? 'c11' : 'c') . '.counter AS counter' .
			($require_stats ? ', t.total AS total' : '') . ' FROM #__sobipro_object AS c JOIN #__sobipro_relations AS cc ON c.id = cc.id ' .
			($isSobiPro11 ? ' LEFT JOIN #__sobipro_counter AS c11 ON c.id = c11.sid ' : '') .
			($require_stats ? ' LEFT OUTER JOIN #__xtdir_totals AS t ON c.id = t.category_id' : '') .
			' WHERE ' . $qparent . ' AND c.oType=' . $db->Quote('category') . $qorder;

		if ($sid_list)
		{
			$query = $query . ' AND c.id IN (' . $sid_list . ')';
		}

		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'Count: ' . $count . ' Query: ' . $query);

		try
		{
			$db->setQuery($query, 0, $count);
			$rows = $db->loadObjectList('id');

			if ($error = $db->getErrorMsg())
			{
				$logger->log(JLog::ERROR, $error);
			}
		}
		catch (Exception $e)
		{
			$logger->log(JLog::ERROR, $e->getMessage());
		}

		$logger->log(JLog::INFO, 'Rows: ', $rows);

		return $rows;
	}

	/**
	 * generate
	 *
	 * @param   array  &$list           Params
	 * @param   int    $iterator_sid    Params
	 * @param   int    $current_sid     Params
	 * @param   int    $my_itemid       Params
	 * @param   array  $sub_Itemids     Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   bool   $hide_empty      Params
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   int    $level           Params
	 * @param   int    $i               Params
	 *
	 * @return  array
	 */
	public static function generate(&$list, $iterator_sid, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0)
	{
		$html_output = array();

		$items = $list[$iterator_sid];

		foreach ($items as $item)
		{
			$evenoddd = ($i % 2 == 0 ? 'even' : 'odd');
			$i++;

			// ItemId
			$item_id = self::getItemId($my_itemid, $sub_Itemids, $item->id, $default_Itemid);

			// Counter
			$counter = $item->counter;
			$total = (property_exists($item, 'total') ? $item->total : 0);
			$c = self::getCounter($counter_type, $total, $counter);
			$output_counter = '';

			if ($c)
			{
				$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
			}

			// It's a parent
			$childs = null;

			if (array_key_exists($item->id, $list))
			{
				$childs = $list[$item->id];
			}

			$class_childs = null;
			$has_childs = false;

			// If ($c > 1)
			if (($childs) && ($level < $end_level))
			{
				$has_childs = true;
				$class_childs = 'hasChilds';
			}

			// Current Page Sid
			$class = ($current_sid == $item->id ? ' active sobiprocateg_active' : null);

			// Leaf
			$ok_to_display = ((($level >= $start_level) && ($level <= $end_level)) && ((!$hide_empty) || ($counter > 0)));

			$output = array();

			if ($ok_to_display)
			{
				// Name
				$category = SobiproHelper::loadAndExtend($item->id);
				$name = $category->get('name');
				$nid = $category->get('nid');

				// Url
				$url_params = array(
								'sid' => $item->id,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);
				$url = htmlspecialchars($url);

				$output[] = "<li id=\"node{$i}\" class=\"sobipcateg_l{$level} {$evenoddd} {$class} {$class_childs}\">";
				$output[] = "<a href=\"{$url}\" class=\"{$class}\">{$name}</a>{$output_counter}\n";
			}

			if ($has_childs)
			{
				$output[] = "<ul class=\"sobipcateg_l{$level} {$evenoddd}{$class}\">" . self::generate($list, $item->id, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level + 1, $i) . '</ul>';
			}

			if ($ok_to_display)
			{
				$output[] = '</li>';
			}

			$html_output[] = join("", $output);
		}

		return join("\n", $html_output);
	}

	/**
	 * generateSelectBox
	 *
	 * @param   array  &$list           Params
	 * @param   int    $iterator_sid    Params
	 * @param   int    $current_sid     Params
	 * @param   int    $my_itemid       Params
	 * @param   array  $sub_Itemids     Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   bool   $hide_empty      Params
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   int    $level           Params
	 * @param   int    $i               Params
	 *
	 * @return  array
	 */
	public static function generateSelectBox(&$list, $iterator_sid, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0)
	{
		$html_output = array();

		$items = $list[$iterator_sid];

		foreach ($items as $item)
		{
			$i++;

			// ItemId
			$item_id = self::getItemId($my_itemid, $sub_Itemids, $item->id, $default_Itemid);

			// Counter
			$counter = $item->counter;
			$total = (property_exists($item, 'total') ? $item->total : 0);
			$c = self::getCounter($counter_type, $total, $counter);
			$output_counter = '';

			if ($c)
			{
				$output_counter = ' (' . $c . ')';
			}

			// If the current item is active
			$isActive = ($current_sid == $item->id);

			// It's a parent
			$childs = null;

			if (array_key_exists($item->id, $list))
			{
				$childs = $list[$item->id];
			}

			// Leaf
			$ok_to_display = ((($level >= $start_level) && ($level <= $end_level)) && ((!$hide_empty) || ($counter > 0)));

			$output = array();

			if ($ok_to_display && (!$childs || $level > 1))
			{
				// Name
				$category = SobiproHelper::loadAndExtend($item->id);
				$name = $category->get('name');
				$nid = $category->get('nid');

				// Url
				$url_params = array(
								'sid' => $item->id,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);
				$url = htmlspecialchars($url);

				$idention = str_repeat('-', $level);
				$output[] = '<option value="' . $url . '"';
				$output[] = $isActive ? ' selected="selected"' : '';
				$output[] = '>' . $idention . $name . $output_counter . '</option>';
			}

			if (($childs) && ($level < $end_level))
			{
				$childrenOutput = self::generateSelectBox($list, $item->id, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level + 1, $i);

				if ($level == 1)
				{
					// Name
					$category = SobiproHelper::loadAndExtend($item->id);
					$name = $category->get('name');

					$output[] = '<optgroup label="' . $name . $output_counter . '">' . $childrenOutput . '</optgroup>';
				}
				else
				{
					$output[] = $childrenOutput;
				}
			}

			$html_output[] = join("", $output);
		}

		return join("\n", $html_output);
	}

	/**
	 * generateChainedCategories
	 *
	 * @param   mixed  $section_id      Params
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   mixed  $query_order     Params
	 * @param   mixed  $count           Params
	 * @param   mixed  $require_stats   Params
	 * @param   int    $current_sid     Params
	 * @param   int    $my_itemid       Params
	 * @param   array  $sub_Itemids     Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   bool   $hide_empty      Params
	 * @param   mixed  $catlist         Params
	 *
	 * @return  string
	 */
	public static function generateChainedCategories($section_id, $start_level, $end_level, $query_order, $count, $require_stats, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $catlist = null)
	{
		$l2Parents = array();
		$l2Parents[] = $section_id;

		$sid_list = null;

		if (count($catlist) > 0)
		{
			$sid_list = implode(',', $catlist);
		}

		$output = array();

		for ($i = 1; ($i <= $end_level); $i++)
		{
			$list = self::_getCategoriesFromParent($l2Parents, $require_stats, $query_order, $count, $sid_list);

			if (is_array($list))
			{
				$l2Parents = array_keys($list);
			}

			if ($i >= $start_level)
			{
				$select = self::_generateChainedCategories($start_level, $end_level, $i, $list, $my_itemid, $default_Itemid, $counter_type, $current_sid, $hide_empty);

				if ($select)
				{
					$output[] = $select;
				}
			}
		}

		return join('', $output);
	}

	/**
	 * _generateChainedCategories
	 *
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   int    $level           Params
	 * @param   array  &$categories     Params
	 * @param   int    $my_itemid       Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   int    $current_sid     Params
	 * @param   bool   $hide_empty      Params
	 *
	 * @return  string
	 */
	private static function _generateChainedCategories($start_level, $end_level, $level, &$categories, $my_itemid, $default_Itemid, $counter_type, $current_sid, $hide_empty)
	{
		if (count($categories) == 0)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, "Empty _getChainedSelect ({$end_level}, {$level}).");

			return null;
		}

		if (($level < $start_level) || ($level > $end_level))
		{
			return '';
		}

		$sid_list = JFactory::getApplication()->input->get('sid_list', null);

		if ((!$sid_list) || (empty($sid_list)))
		{
			$sid_list = null;
			$selected_catlist = null;
		}

		if ($sid_list)
		{
			$selected_catlist = explode(',', self::_cleanListOfNumerics($sid_list));
		}

		$i = 0;
		$output = array();

		if ($end_level == 1)
		{
			$objname = 'sid_list';
		}
		else
		{
			$objname = "to_sid_list_{$level}";
		}

		$first = "<select class='sid_list sid_list{$level}' name='{$objname}'>\n";
		$selected = false;

		foreach ($categories as $category)
		{
			$myId = $category->id;
			$catid = $category->cid;

			// ItemId
			$item_id = self::getItemId($my_itemid, $sub_Itemids, $myId, $default_Itemid);

			// Counter
			$counter = $category->counter;
			$total = (property_exists($category, 'total') ? $category->total : 0);
			$c = self::getCounter($counter_type, $total, $counter);
			$output_counter = '';

			if ($c)
			{
				$output_counter = ' (' . $c . ')';
			}

			// Current Page Sid
			$class = ($current_sid == $myId ? ' active sobiprocateg_active' : null);

			// Leaf
			$ok_to_display = ((!$hide_empty) || ($counter > 0));

			if ((!$selected) && ($selected_catlist) && (in_array($myId, $selected_catlist)))
			{
				$selected = true;
				$selectedhtml = 'selected="selected"';
			}
			else
			{
				$selectedhtml = '';
			}

			if ($end_level == 1)
			{
				$selectValue = $myId;
			}
			else
			{
				$selectValue = 'spc' . $myId;
			}

			if ($ok_to_display)
			{
				// Name
				$ocategory = SobiproHelper::loadAndExtend($myId);
				$categoryName = $ocategory->get('name');
				$categoryNid = $ocategory->get('nid');

				// Url
				$url_params = array(
								'sid' => $myId,
								'title' => (Sobi::Cfg('sef.alias', true) ? $categoryNid : $categoryName)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);
				$url = htmlspecialchars($url);

				$output[] = "<option style=\"\" value=\"{$selectValue}\" class=\"spc{$catid}{$class}\" {$selectedhtml} data-href=\"{$url}\">{$categoryName}{$output_counter}</option>\n";
			}

			$i++;
		}

		$textselect = JText::_('COM_XTDIR_SELECT_CATEGORY_L' . $level);

		if (!$selected)
		{
			$toselect = "<option style=\"\" value=\"\" selected=\"selected\">{$textselect}</option>\n";
		}
		else
		{
			$toselect = "<option style=\"\" value=\"\">{$textselect}</option>\n";
		}

		array_unshift($output, $first, $toselect);

		$output[] = "</select>\n\n";

		return join('', $output);
	}

	/**
	 * getQueryOrder
	 *
	 * @param   mixed  $query_order    Params
	 * @param   mixed  $require_stats  Params
	 *
	 * @return  string
	 */
	public static function getQueryOrder($query_order, $require_stats)
	{
		$qorder = '';

		if ($query_order == 1)
		{
			$qorder = ' ORDER BY c.name';
		}
		elseif ($query_order == 2)
		{
			$qorder = ' ORDER BY c.id';
		}
		elseif ($query_order == 3)
		{
			$qorder = ($require_stats ? ' ORDER BY t.total desc' : ' ORDER BY c.counter desc');
		}
		elseif ($query_order == 4)
		{
			$qorder = ' ORDER BY c.nid';
		}
		elseif ($query_order == 5)
		{
			$qorder = ' ORDER BY cc.position, c.name';
		}
		elseif ($query_order == 6)
		{
			$qorder = ' ORDER BY RAND()';
		}

		return $qorder;
	}

	/**
	 * getDefaultItemid
	 *
	 * @param   int  $parentid  Params
	 *
	 * @return  int
	 */
	public static function getDefaultItemid($parentid)
	{
		$db = JFactory::getDBO();
		$url = $db->quote("index.php?option=com_sobipro&sid=" . $parentid);
		$type = $db->quote('component');

		$query = 'SELECT ' . $db->quoteName('id') . ' FROM ' . $db->quoteName('#__menu') . ' WHERE ' . $db->quoteName('link') . ' = ' . $url . ' AND ' . $db->quoteName('published') . '=' . $db->Quote(1) . ' ' . 'AND ' . $db->quoteName('type') . '=' . $db->Quote('component');
		$db->setQuery($query);
		$defaultId = $db->loadResult();

		return $defaultId;
	}

	/**
	 * getSubItemsid
	 *
	 * @param   int  $parentid  Params
	 *
	 * @return  int
	 */
	public static function getSubItemsid($parentid)
	{
		$db = JFactory::getDBO();
		$url = $db->quote("index.php?option=com_sobipro&sid=%");
		$type = $db->quote('component');

		$query = 'SELECT ' . $db->quoteName('id') . ',' . $db->quoteName('link') . ' FROM ' . $db->quoteName('#__menu') . ' WHERE ' . $db->quoteName('link') . ' like ' . $url . ' AND ' . $db->quoteName('published') . '=' . $db->Quote(1) . ' ' . 'AND ' . $db->quoteName('type') . '=' . $db->Quote('component');

		$db->setQuery($query);
		$results = $db->loadObjectList('id');

		$sub_Itemids = Array();

		foreach ($results as $key => $value)
		{
			$link = $value->link;

			if (preg_match("/sid\=([0-9]+)/", $link, $matches))
			{
				$sub_Itemids[$matches[1]] = $value->id;
			}
		}

		if (isset($sub_Itemids[$parentid]))
		{
			unset($sub_Itemids[$parentid]);
		}

		return $sub_Itemids;
	}

	/**
	 * getItemId
	 *
	 * @param   int    $my_itemid       Params
	 * @param   mixed  &$sub_Itemids    Params
	 * @param   int    $id              Params
	 * @param   int    $default_Itemid  Params
	 *
	 * @return  string
	 */
	public static function getItemId($my_itemid, &$sub_Itemids, $id, $default_Itemid)
	{
		$item_id = null;

		// Force a global itemid
		if (!empty($my_itemid))
		{
			$item_id = $my_itemid;
		}
		else
		{
			if ((!empty($sub_Itemids)) && (count($sub_Itemids) > 0) && (array_key_exists($id, $sub_Itemids)))
			{
				// Try a Sub ItemId
				$item_id = $sub_Itemids[$id];

				if (empty($item_id))
				{
					// Just the default
					$item_id = $default_Itemid;
				}
			}
			else
			{
				// Just the default
				$item_id = $default_Itemid;
			}
		}

		return $item_id;
	}

	/**
	 * getTotal
	 *
	 * @param   int  $counter_type  Params
	 * @param   int  $hits          Params
	 * @param   int  &$totals       Params
	 * @param   int  $catid         Params
	 *
	 * @return  int
	 */
	public static function getTotal($counter_type, $hits, &$totals, $catid)
	{
		if ($counter_type == 0)
		{
			return 0.999;
		}
		// Hits
		elseif ($counter_type == 1)
		{
			return $hits;
		}
		// Entries
		elseif ($counter_type == 2)
		{
			return ($totals[$catid] ? $totals[$catid] : 0);
		}
	}

	/**
	 * getTotal
	 *
	 * @param   int  $counter_type  Params
	 * @param   int  $total         Params
	 * @param   int  $counter       Params
	 *
	 * @return  int
	 */
	public static function getCounter($counter_type, $total, $counter)
	{
		// Hits
		if ($counter_type == 1)
		{
			return $counter;
		}
		// Entries
		if ($counter_type == 2)
		{
			return $total;
		}

		return null;
	}

	/**
	 * sortCategories
	 *
	 * @param   array  $list          Params
	 * @param   int    $section_id    Params
	 * @param   int    $counter_type  Params
	 * @param   bool   $hide_empty    Params
	 * @param   int    $start_level   Params
	 * @param   int    $end_level     Params
	 * @param   int    $count         Params
	 *
	 * @return  array
	 */
	public static function sortCategories($list, $section_id, $counter_type, $hide_empty, $start_level, $end_level, $count = 100)
	{
		$list = self::_sortCategories($list, $section_id, $counter_type, $hide_empty, $start_level, $end_level);
		usort($list, create_function('$a, $b', 'return ($a->c < $b->c);'));
		$list = array_slice($list, 0, $count);

		return array(
						$section_id => $list
		);
	}

	/**
	 * _sortCategories
	 *
	 * @param   array  $list          Params
	 * @param   int    $iterator_sid  Params
	 * @param   int    $counter_type  Params
	 * @param   bool   $hide_empty    Params
	 * @param   int    $start_level   Params
	 * @param   int    $end_level     Params
	 * @param   int    $level         Params
	 * @param   int    $i             Params
	 *
	 * @return  array
	 */
	private static function _sortCategories($list, $iterator_sid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0)
	{
		$output = array();

		$items = $list[$iterator_sid];

		foreach ($items as $item)
		{
			$i++;

			// Counter
			$counter = $item->counter;
			$total = (property_exists($item, 'total') ? $item->total : 0);
			$c = self::getCounter($counter_type, $total, $counter);

			// Leaf
			$ok_to_display = ((($level >= $start_level) && ($level <= $end_level)) && ((!$hide_empty) || ($c > 0)));

			if ($ok_to_display)
			{
				$item->c = $c;
				$output[] = $item;
			}

			// It's a parent
			$childs = null;

			if (array_key_exists($item->id, $list))
			{
				$childs = $list[$item->id];
			}

			if (($childs) && ($level < $end_level))
			{
				$o = self::_sortCategories($list, $item->id, $counter_type, $hide_empty, $start_level, $end_level, $level + 1, $i);
				$output = array_merge($output, $o);
			}
		}

		return $output;
	}

	/**
	 * getChainedCategories
	 *
	 * @param   int     $section      Params
	 * @param   int     $start_level  Params
	 * @param   int     $end_level    Params
	 * @param   string  $query_order  Params
	 * @param   array   $catlist      Params
	 *
	 * @return  string
	 */
	public static function getChainedCategoriesv1($section, $start_level, $end_level, $query_order, $catlist = null)
	{
		$l2Parents = array();
		$l2Parents[] = $section;

		$sid_list = null;

		if (count($catlist) > 0)
		{
			$sid_list = implode(',', $catlist);
		}

		$output = array();

		for ($i = 1; ($i <= $end_level); $i++)
		{
			$list = self::_getChainedListv1($l2Parents, $query_order, $sid_list);

			if (is_array($list))
			{
				$l2Parents = array_keys($list);
			}

			if ($i >= $start_level)
			{
				$select = self::_getChainedSelectv1($end_level, $i, $list);

				if ($select)
				{
					$output[] = $select;
				}
			}
		}

		return join('', $output);
	}

	/**
	 * _getChainedList
	 *
	 * @param   mixed  $parent    the params
	 * @param   mixed  $sorder    the params
	 * @param   mixed  $sid_list  the params
	 *
	 * @return  array
	 */
	private static function _getChainedListv1($parent, $sorder, $sid_list)
	{
		$lparent = join(',', $parent);
		$db = JFactory::getDBO();
		$qorder = '';

		if ($sorder == 1)
		{
			$qorder = ' ORDER BY c.name';
		}
		elseif ($sorder == 2)
		{
			$qorder = ' ORDER BY c.id';
		}
		elseif ($sorder == 3)
		{
			$qorder = ' ORDER BY c.counter desc';
		}
		elseif ($sorder == 4)
		{
			$qorder = ' ORDER BY c.nid';
		}
		elseif ($sorder == 5)
			$qorder = ' ORDER BY cc.position, c.name';

		$query = 'SELECT c.id AS id, c.name AS name, c.nid AS alias, c.counter AS counter, c.parent as cid' . ' FROM #__sobipro_object AS c ' . ' LEFT JOIN #__sobipro_relations AS cc ON c.id = cc.id ' . ' WHERE c.state = 1 AND c.oType=' . $db->Quote('category');

		$query = $query . ' AND parent IN (' . $lparent . ')';

		if ($sid_list)
		{
			$query = $query . ' AND c.id IN (' . $sid_list . ')';
		}

		$query = $query . $qorder;

		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'QUERY:' . $query);

		$db->setQuery($query);
		$rows = $db->loadObjectList('id');

		if ($rows)
		{
			$logger->log(JLog::INFO, "Rows parent ({$lparent}):" . count($rows));
		}
		else
		{
			$logger->log(JLog::INFO, "No rows({$lparent}).");
		}

		return $rows;
	}

	/**
	 * _getChainedSelectv1
	 *
	 * @param   int    $end_level    Params
	 * @param   int    $level        Params
	 * @param   array  &$categories  Params
	 *
	 * @return  array
	 */
	private static function _getChainedSelectv1($end_level, $level, &$categories)
	{
		if (count($categories) == 0)
		{
			$logger = XtdirLogger::getInstance();
			$logger->log(JLog::INFO, "Empty _getChainedSelect ({$end_level}, {$level}).");

			return null;
		}

		$sid_list = JFactory::getApplication()->input->get('sid_list', null);

		if ((!$sid_list) || (empty($sid_list)))
		{
			$sid_list = null;
			$selected_catlist = null;
		}

		if ($sid_list)
		{
			$selected_catlist = explode(',', self::_cleanListOfNumerics($sid_list));
		}

		$i = 0;
		$output = array();

		if ($end_level == 1)
		{
			$objname = 'sid_list';
		}
		else
		{
			$objname = "to_sid_list_{$level}";
		}

		$first = "<select class='sid_list sid_list{$level}' name='{$objname}'>";
		$selected = false;

		foreach ($categories as $category)
		{
			$catid = $category->cid;
			$myId = $category->id;

			if ((!$selected) && ($selected_catlist) && (in_array($myId, $selected_catlist)))
			{
				$selected = true;
				$selectedhtml = 'selected="selected"';
			}
			else
			{
				$selectedhtml = '';
			}

			// Name
			$ocategory = SobiproHelper::loadAndExtend($myId);
			$category_name = $ocategory->get('name');

			if ($end_level == 1)
			{
				$selectValue = $myId;
			}
			else
			{
				$selectValue = 'spc' . $myId;
			}

			$output[] = "<option style=\"\" value=\"{$selectValue}\" class=\"spc{$catid}\" {$selectedhtml}>{$category_name}</option>";
			$i++;
		}

		$textselect = JText::_('COM_XTDIR_SELECT_CATEGORY_L' . $level);

		if (!$selected)
		{
			$toselect = "<option style=\"\" value=\"\" selected=\"selected\">{$textselect}</option>";
		}
		else
		{
			$toselect = "<option style=\"\" value=\"\">{$textselect}</option>";
		}

		array_unshift($output, $first, $toselect);

		$output[] = '</select>';

		return join('', $output);
	}

	/**
	 * _cleanListOfNumerics
	 *
	 * @param   string  $listOfNumerics  the params
	 *
	 * @return  string
	 */
	private static function _cleanListOfNumerics($listOfNumerics)
	{
		return preg_replace('/[^,0-9]/', '', $listOfNumerics);
	}

	/**
	 * getCurrentSid
	 *
	 * @return  int
	 */
	public static function getCurrentSid()
	{
		// Current Url, SID
		$raw_sid_full = JFactory::getApplication()->input->get('sid');
		$current_sid_full = explode(":", $raw_sid_full);
		$current_sid = (count($current_sid_full) == 2 ? $current_sid_full[0] : null);

		return $current_sid;
	}

	/**
	 * getCurrentSid
	 *
	 * @return  array
	 */
	public static function getSelectedCategories()
	{
		$sid_list = JFactory::getApplication()->input->get('sid_list', null);

		if ((!$sid_list) || (empty($sid_list)))
		{
			$sid_list = null;
			$selected_catlist = null;
		}

		if ($sid_list)
		{
			$selected_catlist = explode(',', self::_cleanListOfNumerics($sid_list));
		}

		return $selected_catlist;
	}

	/**
	 * getCategoryNid
	 *
	 * @param   array  &$fields  Params
	 *
	 * @return  string
	 */
	public static function getCategoryNid(&$fields)
	{
		foreach ($fields as $field)
		{
			if ($field->get('fieldType') == 'category')
			{
				return $field->get('nid');
			}
		}

		return 'field_select_category';
	}

	/**
	 * array_flatten
	 *
	 * @param   array  $list  Params
	 * @param   array  $root  Params
	 *
	 * @return  string
	 */
	public static function array_flatten($list, $root)
	{
		if (!array_key_exists($root->id, $list) || !$list[$root->id])
		{
			return array();
		}

		$childs = $list[$root->id];
		$result[] = $root;

		foreach ($childs as $child)
		{
			$intermedio = self::array_flatten($list, $child);
			$result = array_merge($result, $intermedio);
		}

		return $result;
	}

	/**
	 * generateGroupingByCategory
	 *
	 * @param   array  &$list           Params
	 * @param   int    $iterator_sid    Params
	 * @param   int    $current_sid     Params
	 * @param   int    $my_itemid       Params
	 * @param   array  $sub_Itemids     Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   bool   $hide_empty      Params
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   int    $level           Params
	 * @param   int    $i               Params
	 * @param   int    $columns         Params
	 *
	 * @return  string
	 */
	public static function generateGroupingByCategory(&$list, $iterator_sid, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0, $columns = 2)
	{
		$html_output = array();

		$firstLevel = $list[$iterator_sid];

		$span = 12 / $columns;
		$j = 0;
		$rowOpen = false;

		foreach ($firstLevel as $itemOfFirstLevel)
		{
			$flatten = self::array_flatten($list, $itemOfFirstLevel);

			foreach ($flatten as $item)
			{
				$i++;

				// ItemId
				$item_id = self::getItemId($my_itemid, $sub_Itemids, $item->id, $default_Itemid);

				// Counter
				$counter = $item->counter;
				$total = (property_exists($item, 'total') ? $item->total : 0);
				$c = self::getCounter($counter_type, $total, $counter);
				$output_counter = '';

				if ($c)
				{
					$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
				}

				// It's a parent
				$childs = null;

				if (array_key_exists($item->id, $list))
				{
					$childs = $list[$item->id];
				}

				$has_childs = false;

				// If ($c > 0)
				if ($childs)
				{
					$has_childs = true;
				}

				$output = array();

				// Open Column / Row
				if ($span != 12)
				{
					$isRow = ($j % 12 == 0);

					if ($isRow)
					{
						$rowOpen = true;
						$output[] = "\n<div class=\"row-fluid\">";
					}

					$output[] = "\n<div class=\"span" . $span . '">';
				}

				// Name
				$category = SobiproHelper::loadAndExtend($item->id);
				$name = $category->get('name');
				$nid = $category->get('nid');

				// Url
				$url_params = array(
								'sid' => $item->id,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);
				$url = htmlspecialchars($url);

				$output[] = '<dl>';
				$output[] = "\n<dt><a href=\"{$url}\"><strong>{$name}</strong></a>{$output_counter}</dt>";

				if ($has_childs)
				{
					$output[] = "\n<dd><ul>";

					foreach ($childs as $item)
					{
						// Name
						$category = SobiproHelper::loadAndExtend($item->id);
						$name = $category->get('name');
						$nid = $category->get('nid');

						// Counter
						$counter = $item->counter;
						$total = (property_exists($item, 'total') ? $item->total : 0);
						$c = self::getCounter($counter_type, $total, $counter);
						$output_counter = '';

						if ($c)
						{
							$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
						}

						// Url
						$url_params = array(
										'sid' => $item->id,
										'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
						);

						if ($item_id)
						{
							$url_params['Itemid'] = $item_id;
						}

						$url = Sobi::Url($url_params);
						$url = htmlspecialchars($url);

						$output[] = "\n<li>";
						$output[] = "<a href=\"{$url}\">{$name}</a>{$output_counter}\n";
						$output[] = '</li>';
					}

					$output[] = "\n</ul></dd>";
				}

				$output[] = "\n</dl>";

				// Close Column / Row
				if ($span != 12)
				{
					$output[] = "\n</div>";
					$j += $span;

					if ($j % 12 == 0)
					{
						$output[] = "\n</div>";
						$rowOpen = false;
					}
				}

				$html_output[] = join("", $output);
			}
		}

		if ($rowOpen)
		{
			$html_output[] = "\n</div>";
			$rowOpen = false;
		}

		return join("\n", $html_output);
	}

	/**
	 * generateGroupingByCategoryWithImage
	 *
	 * @param   array  &$list           Params
	 * @param   int    $iterator_sid    Params
	 * @param   int    $current_sid     Params
	 * @param   int    $my_itemid       Params
	 * @param   array  $sub_Itemids     Params
	 * @param   int    $default_Itemid  Params
	 * @param   int    $counter_type    Params
	 * @param   bool   $hide_empty      Params
	 * @param   int    $start_level     Params
	 * @param   int    $end_level       Params
	 * @param   int    $level           Params
	 * @param   int    $i               Params
	 * @param   int    $columns         Params
	 *
	 * @return  string
	 */
	public static function generateGroupingByCategoryWithImage(&$list, $iterator_sid, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0, $columns = 2)
	{
		$html_output = array();

		$firstLevel = $list[$iterator_sid];

		$span = 12 / $columns;
		$j = 0;
		$rowOpen = false;

		$category_icons = Sobi::Cfg('images.category_icons');

		foreach ($firstLevel as $itemOfFirstLevel)
		{
			$flatten = self::array_flatten($list, $itemOfFirstLevel);

			foreach ($flatten as $item)
			{
				$i++;

				// ItemId
				$item_id = self::getItemId($my_itemid, $sub_Itemids, $item->id, $default_Itemid);

				// Counter
				$counter = $item->counter;
				$total = (property_exists($item, 'total') ? $item->total : 0);
				$c = self::getCounter($counter_type, $total, $counter);
				$output_counter = '';

				if ($c)
				{
					$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
				}

				// It's a parent
				$childs = null;

				if (array_key_exists($item->id, $list))
				{
					$childs = $list[$item->id];
				}

				$has_childs = false;

				// If ($c > 0)
				if ($childs)
				{
					$has_childs = true;
				}

				$output = array();

				// Open Column / Row
				if ($span != 12)
				{
					$isRow = ($j % 12 == 0);

					if ($isRow)
					{
						$rowOpen = true;
						$output[] = "\n<div class=\"row-fluid\">";
					}

					$output[] = "\n<div class=\"span" . $span . '">';
				}

				// Name
				$category = SobiproHelper::loadAndExtend($item->id);
				$name = $category->get('name');
				$nid = $category->get('nid');
				$description = $category->get('description');
				$icon = $category->get('icon');

				if (SPFs::exists($category_icons . '/' . $icon))
				{
					$image = Sobi::FixPath(Sobi::Cfg('images.category_icons_live') . $icon);
				}
				else
				{
					$image = 'media/mod_categories/images/folder.png';
				}

				// Url
				$url_params = array(
								'sid' => $item->id,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);
				$url = htmlspecialchars($url);

				$output[] = "\n<div class=\"xt-media-parent media\">";

				$output[] = "<a href=\"{$url}\" class=\"pull-left\">";
				$output[] = '<img class="media-object" src="' . $image . '">';
				$output[] = "</a>";

				$output[] = '<div class="media-body">';
				$output[] = "<a href=\"{$url}\" class='media-heading'><strong>{$name} {$output_counter}</strong></a>\n";
				$output[] = $description . "\n";

				if ($has_childs)
				{
					foreach ($childs as $item)
					{
						// Name
						$category = SobiproHelper::loadAndExtend($item->id);
						$name = $category->get('name');
						$nid = $category->get('nid');
						$description = $category->get('description');
						$icon = $category->get('icon');

						if (SPFs::exists($category_icons . '/' . $icon))
						{
							$image = Sobi::FixPath(Sobi::Cfg('images.category_icons_live') . $icon);
						}
						else
						{
							$image = 'media/mod_categories/images/folder.png';
						}

						// Counter
						$counter = $item->counter;
						$total = (property_exists($item, 'total') ? $item->total : 0);
						$c = self::getCounter($counter_type, $total, $counter);
						$output_counter = '';

						if ($c)
						{
							$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
						}

						// Url
						$url_params = array(
										'sid' => $item->id,
										'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
						);

						if ($item_id)
						{
							$url_params['Itemid'] = $item_id;
						}

						$url = Sobi::Url($url_params);
						$url = htmlspecialchars($url);

						$output[] = "\n<div class=\"xt-media-child media\">";

						$output[] = "<a href=\"{$url}\" class=\"pull-left\">";
						$output[] = '<img class="media-object" src="' . $image . '">';
						$output[] = "</a>";

						$output[] = '<div class="media-body">';
						$output[] = "<a href=\"{$url}\" class='media-heading'>{$name} {$output_counter}</a>\n";
						$output[] = $description . "\n";
						$output[] = '</div>';

						$output[] = '</div>';
					}
				}

				$output[] = "\n</div>";
				$output[] = "\n</div>";

				// Close Column / Row
				if ($span != 12)
				{
					$output[] = "\n</div>";
					$j += $span;

					if ($j % 12 == 0)
					{
						$output[] = "\n</div>";
						$rowOpen = false;
					}
				}

				$html_output[] = join("", $output);
			}
		}

		if ($rowOpen)
		{
			$html_output[] = "\n</div>";
			$rowOpen = false;
		}

		return join("\n", $html_output);
	}

	/**
	 * generateAccordionCategories
	 *
	 * @param   array   &$list           Params
	 * @param   int     $iterator_sid    Params
	 * @param   int     $current_sid     Params
	 * @param   int     $my_itemid       Params
	 * @param   array   $sub_Itemids     Params
	 * @param   int     $default_Itemid  Params
	 * @param   int     $counter_type    Params
	 * @param   bool    $hide_empty      Params
	 * @param   int     $start_level     Params
	 * @param   int     $end_level       Params
	 * @param   int     $level           Params
	 * @param   int     $i               Params
	 * @param   string  $moduleId        Params
	 *
	 * @return  string
	 */
	public static function generateAccordionCategories(&$list, $iterator_sid, $current_sid, $my_itemid, $sub_Itemids, $default_Itemid, $counter_type, $hide_empty, $start_level, $end_level, $level = 1, $i = 0, $moduleId = null)
	{
		$html_output = array();

		$firstLevel = $list[$iterator_sid];
		$parentList = array();

		foreach ($firstLevel as $itemOfFirstLevel)
		{
			$flatten = self::array_flatten($list, $itemOfFirstLevel);

			foreach ($flatten as $item)
			{
				$i++;

				// ItemId
				$item_id = self::getItemId($my_itemid, $sub_Itemids, $item->id, $default_Itemid);

				// Counter
				$counter = $item->counter;
				$total = (property_exists($item, 'total') ? $item->total : 0);
				$c = self::getCounter($counter_type, $total, $counter);
				$output_counter = '';

				if ($c)
				{
					$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
				}

				// It's a parent
				if (array_key_exists($item->id, $list))
				{
					$childs = $list[$item->id];
				}

				$class_childs = null;
				$has_childs = false;

				// If ($c > 1)
				if ($childs)
				{
					$has_childs = true;
					$class_childs = 'hasChilds';
				}

				$output = array();

				// Name
				$category = SobiproHelper::loadAndExtend($item->id);
				$name = $category->get('name');
				$nid = $category->get('nid');

				// Url
				$url_params = array(
								'sid' => $item->id,
								'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
				);

				if ($item_id)
				{
					$url_params['Itemid'] = $item_id;
				}

				$url = Sobi::Url($url_params);

				$output[] = '<div class="accordion-group">';

				$output[] = '<div class="accordion-heading">';
				$output[] = "<a class=\"accordion-toggle\" data-toggle=\"collapse\" data-parent=\"#{$moduleId}\" href=\"#collapse{$moduleId}-{$i}\">";
				$output[] = $name . $output_counter;
				$output[] = '</a>';
				$output[] = '</div>';

				if ($has_childs)
				{
					if ($i == 1)
					{
						$output[] = "<div id=\"collapse{$moduleId}-{$i}\" class=\"accordion-body collapse in\">";
					}
					else
					{
						$output[] = "<div id=\"collapse{$moduleId}-{$i}\" class=\"accordion-body collapse\">";
					}

					$output[] = '<div class="accordion-inner">';
					$output[] = '<ul>';

					foreach ($childs as $item)
					{
						// Counter
						$counter = $item->counter;
						$total = (property_exists($item, 'total') ? $item->total : 0);
						$c = self::getCounter($counter_type, $total, $counter);
						$output_counter = '';

						if ($c)
						{
							$output_counter = ' <span class="xtd-counter"> (' . $c . ')</span>';
						}

						// Name
						$category = SobiproHelper::loadAndExtend($item->id);
						$name = $category->get('name');
						$nid = $category->get('nid');
						$image = $category->get('icon');

						// Url
						$url_params = array(
							'sid' => $item->id,
							'title' => (Sobi::Cfg('sef.alias', true) ? $nid : $name)
						);

						if ($item_id)
						{
							$url_params['Itemid'] = $item_id;
						}

						$url = Sobi::Url($url_params);
						$url = htmlspecialchars($url);

						$output[] = "<li>";
						$output[] = "<a href=\"{$url}\">{$name}</a>{$output_counter}\n";
						$output[] = '</li>';
					}

					$output[] = '</ul>';
					$output[] = '</div>';
					$output[] = '</div>';
				}

				$output[] = '</div>';

				$html_output[] = join("", $output);
			}
		}

		return join("\n", $html_output);
	}
}
