<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';

/**
 * ExtendedSearchHelper
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class ExtendedSearchHelper
{
	/**
	 * generateOutput
	 *
	 * @param   array   &$fields            Params
	 * @param   string  &$categoriesSelect  Params
	 * @param   int     $uniqueId           Params
	 *
	 * @return  string
	 *
	 * @since   1.0
	 */
	public static function generateOutput(&$fields, &$categoriesSelect, $uniqueId = null)
	{
		if (empty($uniqueId))
		{
			$uniqueId = rand(1, 10000);
		}

		$fieldsOutput = array();
		$tab = "\t";

		$fieldsJs = array();

		if (count($fields) > 0)
		{
			foreach ($fields as $field)
			{
				$nid = $field->get('nid');
				$name = $field->get('name');
				$id = $field->get('id');
				$type = $field->get('type');
				$position = $field->get('position');
				$searchMethod = $field->get('searchMethod');

				if ($searchMethod == 'general')
				{
					continue;
				}

				$xtclass = 'xtd' . $type . ' input-medium';

				if (strlen($field->get('cssClass')))
				{
					$css_class = $field->get('cssClass');
					$css_class .= ' ' . $xtclass;
					$field->set('cssClass', $css_class);
				}
				else
				{
					$field->set('cssClass', $xtclass);
				}

				$css_class = $field->get('cssClass');

				if ($type == 'category')
				{
					if ($categoriesSelect)
					{
						$data = $categoriesSelect;
					}
					else
					{
						$data = $field->searchForm();
					}

					if ($field->get('searchMethod') == 'select')
					{
						$data = str_replace($nid, 'sid_list', $data);
					}
				}
				else
				{
					$data = $field->searchForm();
				}

				// Cleaning fixed styles
				$pattern = "/(style=\"[^\"]+\")/si";
				$data = preg_replace($pattern, '', $data);

				if ($type == 'calendar')
				{
					$nid = $field->get('nid') . $uniqueId;
					$jsCode = SobiProHelper::addTimepicker($nid, $field);

					if ($jsCode)
					{
						$fieldsJs[] = $jsCode;
					}

					$data = preg_replace('/id="([a-z0-9_]+)(_from|_to)(_selector)?"/', 'id="${1}' . $uniqueId . '$2$3"', $data);
					$data = str_replace('readonly="readonly"', 'readonly="readonly" style="display:none"', $data);
				}

				$suffix = $field->get('searchMethod') != 'range' ? $field->get('suffix') : null;

				$fieldsOutput[] = '<div class="control-group group-' . $type . ' group-' . $nid . '">';
				$fieldsOutput[] = $tab . '<label class="control-label" for="' . $nid . '">';
				$fieldsOutput[] = $tab . $tab . $name;
				$fieldsOutput[] = $tab . '</label>';
				$fieldsOutput[] = $tab . '<div class="controls">';
				$fieldsOutput[] = $tab . $tab . '<div' . (strlen($suffix) ? ' class="' . $suffix . '"' : '') . '>';
				$fieldsOutput[] = $tab . $tab . $tab . $data;

				if (strlen($suffix))
				{
					$fieldsOutput[] = $tab . $tab . $tab . '<span class="add-on">';
					$fieldsOutput[] = $tab . $tab . $tab . $tab . $suffix;
					$fieldsOutput[] = $tab . $tab . $tab . '</span>';
				}

				$fieldsOutput[] = $tab . $tab . '</div>';
				$fieldsOutput[] = $tab . '</div>';
				$fieldsOutput[] = '</div>';
			}
		}

		if (!empty($fieldsJs))
		{
			Extly::addPostRequireScript(implode('', $fieldsJs));

			// $statement = 'jQuery(".xtdcalendar").parent().find(":input[readonly=\'readonly\']").addClass("hide");';
			// Extly::addPostRequireScript($statement);
		}

		return join("\n", $fieldsOutput);
	}

	/**
	 * session
	 *
	 * @param   string  &$ssid  Params
	 *
	 * @return  bool
	 *
	 * @since   1.0
	 */
	public static function session( &$ssid )
	{
		/* if it wasn't new search */
		$ssid = SPRequest::cmd('ssid', SPRequest::cmd('ssid', null, 'cookie'));
		$new = false;

		/* otherwise create new ssid */
		if (!$ssid)
		{
			$ssid = (microtime(true) * 100) . '.' . rand(0, 99);
			$new = true;
		}

		if (!class_exists('SPBrowser'))
		{
			SPLoader::loadClass('env.browser');
		}

		$attr = array(
				'ssid' => $ssid,
				'uid' => Sobi::My('id'),
				'browserData' => SPConfig::serialize(SPBrowser::getInstance())
		);

		/* get search request - No query yet */
		/*if ( !( count( $this->_request ) ) ) {
			$r = SPRequest::search( 'field_' );
		if ( is_array( $r ) && count( $r ) ) {
		$attr[ 'requestData' ] = SPConfig::serialize( $r );
		}
		}*/

		/* determine the search parameters */
		if ($new)
		{
			$attr[ 'searchCreated' ] = 'FUNCTION:NOW()';
		}

		/* finally save */
		try
		{
			$db = SPFactory::db();
			$db->insertUpdate('spdb_search', $attr);
		}
		catch (SPException $x)
		{
			Sobi::Error('ExtendedSearchHelper::session', SPLang::e('CANNOT_CREATE_SESSION_DB_ERR', $x->getMessage()), SPC::ERROR, 500, __LINE__, __FILE__);
		}

		if (!class_exists('SPCookie'))
		{
			SPLoader::loadClass('env.cookie');
		}

		return SPCookie::set('ssid', $ssid, SPCookie::days(7));
	}

	/**
	 * initApp - Init JS Libraries
	 *
	 * @param   string  $version             Params
	 * @param   string  $jsapps              Params
	 * @param   array   $dependencies        Params
	 * @param   array   $paths               Params
	 * @param   bool    $disable_own_jquery  Params
	 * @param   bool    $form_autosave       Params
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public static function initApp($version, $jsapps, $dependencies = array(), $paths = array(), $disable_own_jquery = false, $form_autosave = true)
	{
		Extly::getScriptManager(
			// LoadExtlyAdminMode
			false,
			// Own Jquery Disabled
			$disable_own_jquery,
			// LoadBootstrap
			false
		);

		$appName = Extly::getAppname($jsapps);

		if ($form_autosave)
		{
			$dependencies[$appName] = array('extlycore', 'xtdhelper', 'chained', 'saveform');
		}
		else
		{
			$dependencies[$appName] = array('extlycore', 'xtdhelper', 'chained');
		}

		if (EXTLY_J25)
		{
			if (!$disable_own_jquery)
			{
				$dependencies['chained'] = array('jquery1102');

				if ($form_autosave)
				{
					$dependencies['saveform'] = array('jquery1102');
				}
			}
		}

		$paths['chained'] = 'media/com_xtdir/js/jquery.chained.min';
		$paths['xtdhelper'] = 'media/com_xtdir/js/xtdhelper.min';

		if ($form_autosave)
		{
			$paths['saveform'] = Extly::JS_LIB . 'utils/jquery.xtsaveform.min';
		}

		$dependencies['xtdhelper'] = array('backbone');

		Extly::initApp(CXTDIR_VERSION, $jsapps, $dependencies, $paths);

		$urlroot = JUri::root();
		$sobiProUrl = "window.SobiProUrl = '{$urlroot}index.php?option=com_sobipro&task=%task%';";
		$document = JFactory::getDocument();
		$document->addScriptDeclaration($sobiProUrl);
	}
}
