<?php
/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * SelectControlHelper
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class SelectControlHelper
{
	/**
	 * sectionlist.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   array   $config    Config
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function sectionlist($selected = null, $name = 'sectionlist', $attribs = array(), $config = null)
	{
		$sectionsModel = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$sectionsModel->setState('state', 1);
		$sectionsModel->setState('oType', 'section');
		$sectionsModel->setState('filter_order', 'name');
		$items = $sectionsModel->getItemList(true);

		$options = array();
		EHtmlSelect::addDefaultSelectOption($attribs, $options);

		if (count($items))
		{
			foreach ($items as $item)
			{
				$options[] = JHTML::_('select.option', $item->id, $item->name);
			}
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * fieldlist.
	 *
	 * @param   string  $selected   The key that is selected
	 * @param   string  $name       The name for the field
	 * @param   array   $attribs    Additional HTML attributes for the <select> tag
	 * @param   array   $config     Config
	 * @param   string  $idTag      Param
	 * @param   int     $section    Param
	 * @param   string  $fieldtype  Param
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function fieldlist($selected = null, $name = 'fieldlist', $attribs = array(), $config = null, $idTag = 'fieldlist', $section = null, $fieldtype = null)
	{
		$fieldsModel = FOFModel::getTmpInstance('Fields', 'XtdirModel');
		$fieldsModel->setState('enabled', 1);
		$fieldsModel->setState('filter_order', 'section,position');

		$sections = null;

		if ($section)
		{
			$fieldsModel->setState('section', $section);
		}
		else
		{
			$catalog_list = EExtensionHelper::getExtensionParam('system', 'xtdir', 'catalog_list');
			$c = count($catalog_list);

			if ($c)
			{
				$sections = implode(',', $catalog_list);

				// Optimization for just 1 section
				if ($c == 1)
				{
					$section = $sections;
					$sections = null;
					$fieldsModel->setState('section', $section);
				}
				else
				{
					$fieldsModel->setState('sections', $sections);
				}
			}
		}

		if ($fieldtype)
		{
			$fieldsModel->setState('fieldType', $fieldtype);
		}

		$items = $fieldsModel->getItemList(true);

		$options = array();
		EHtmlSelect::addDefaultSelectOption($attribs, $options);

		if (count($items))
		{
			foreach ($items as $item)
			{
				if ($sections)
				{
					$object = SPFactory::object($item->section);
					$fname = $object->name . ' - ' . $item->nid;
				}
				else
				{
					$fname = $item->nid;
				}

				$options[] = JHTML::_('select.option', $item->fid, $fname);
			}
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * categorylist.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   string  $idTag     Param
	 * @param   int     $section   Param
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function categorylist($selected = null, $name = 'categorylist', $attribs = array(), $idTag = 'categorylist', $section = null)
	{
		$truncate_cats = EParameter::getComponentParam(CXTDIR, 'truncate_cats', UtilHelper::CATS_TEXT_LENGHT);

		$categoriesModel = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$categoriesModel->setState('state', 1);
		$categoriesModel->setState('oType', 'category');

		if ($section)
		{
			$categoriesModel->setState('section_id', $section);
		}

		$categoriesModel->setState('filter_order', 'name');
		$items = $categoriesModel->getItemList(true);

		$options = array();
		EHtmlSelect::addDefaultSelectOption($attribs, $options);

		if (count($items))
		{
			foreach ($items as $item)
			{
				$parent = SPFactory::object($item->parent);

				$v = $item->name;

				if ($truncate_cats)
				{
					$v = UtilHelper::truncString($v, $truncate_cats, true);
				}

				$options[] = JHTML::_('select.option', $item->id, $v);

				// $options[] = JHTML::_('select.option', $item->id, $parent->name . ' - ' . $item->name);
			}
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * menuitemlist.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   array   $config    Config
	 * @param   string  $idTag     Param
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function menuitemlist($selected = null, $name = 'fieldlist', $attribs = array(), $config = null, $idTag = 'fieldlist')
	{
		if (EXTLY_J3)
		{
			$menuitem = JFormHelper::loadFieldType('Menuitem', false);

			$custom = '';

			if (count($attribs) > 0)
			{
				foreach ($attribs as $key => $value)
				{
					$custom = $custom . ' ' . $key . '="' . $value . '" ';
				}
			}

			$element = new SimpleXMLElement('<field name="' . $name . '" type="menuitem" default="" size="7"' . $custom . '><option value="">' . JText::_('COM_XTDIR_SELECT') . '</option></field>');
			$menuitem->setup($element, $selected);

			return $menuitem->input;
		}
		else
		{
			$mclass = null;

			if (array_key_exists('class', $attribs))
			{
				$mclass = 'class="' . $attribs['class'] . '"';
			}

			$control = "<input type=\"text\" name=\"{$name}\" id=\"{$idTag}\" value=\"{$selected}\" {$mclass}/>";

			return $control;
		}
	}

	/**
	 * tiertypelist.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   array   $config    Config
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function tiertypelist($selected = null, $name = 'tiertypelist', $attribs = array(), $config = null)
	{
		$sectionsModel = FOFModel::getTmpInstance('Tiertypes', 'XtdirModel');
		$sectionsModel->setState('published', 1);
		$sectionsModel->setState('filter_order', 'name');
		$items = $sectionsModel->getItemList(true);

		$options = array();
		EHtmlSelect::addDefaultSelectOption($attribs, $options);

		if (count($items))
		{
			foreach ($items as $item)
			{
				$options[] = JHTML::_('select.option', $item->id, JText::_($item->name));
			}
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * promotypeBtngroup.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function promotypeBtngroup($selected = null, $name = 'promotype_id', $attribs = array())
	{
		$options = array();
		$options[] = array(
						'name' => '-',
						'value' => 0
		);
		$options[] = array(
						'name' => '<i class="xticon xticon-star"></i>&nbsp;',
						'value' => ord('M')
		);
		$options[] = array(
						'name' => '<i class="xticon xticon-star-o"></i>&nbsp;',
						'value' => ord('A')
		);
		$options[] = array(
						'name' => '<i class="xticon xticon-circle-o"></i>&nbsp;',
						'value' => ord('N')
		);
		$btn = EHtmlSelect::btnGroupList($selected, $name, $attribs, $options);

		return $btn;
	}

	/**
	 * jsAppPositionsList.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function jsAppPositionsList($selected = null, $name = 'position', $attribs = array())
	{
		$options = array();

		$options[] = JHTML::_('select.option', 'content', 'content');
		$options[] = JHTML::_('select.option', 'sidebar-top', 'sidebar-top');
		$options[] = JHTML::_('select.option', 'sidebar-bottom', 'sidebar-bottom');

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * levels.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function levels($selected = 2, $name = 'name_id', $attribs = array() )
	{
		$options = array();

		// Generate html
		for ($i = 1; ($i < 10); $i++)
		{
			$options[] = JHtml::_('select.option', $i, $i, 'value', 'text');
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * queryOrders.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function queryOrders($selected = null, $name = 'name_id', $attribs = array() )
	{
		$options = array();

		$orders = array(
			'0' => 'Default - no order',
			'1' => 'Alphabetically',
			'2' => 'Id',
			'3' => 'Counter Desc',
			'5' => 'Position',
			'6' => 'Random',
			'7' => 'Hit Counter Desc',
			'4' => 'Alphabetically (nid)'
		);

		// Generate html
		foreach ($orders as $k => $v)
		{
			$options[] = JHtml::_('select.option', $k, $v, 'value', 'text');
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * counterTypes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function counterTypes($selected = null, $name = 'name_id', $attribs = array() )
	{
		$options = array();

		$orders = array(
						'0' => 'JNO',
						'1' => 'Hits',
						'2' => 'Entries'
		);

		// Generate html
		foreach ($orders as $k => $v)
		{
			$options[] = JHtml::_('select.option', $k, JText::_($v), 'value', 'text');
		}

		return EHtmlSelect::customGenericList($options, $name, $attribs, $selected, $name);
	}

	/**
	 * yesNoSpApp.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function yesNoSpApp($selected = null, $name = 'name_id', $attribs = array())
	{
		return EHtmlSelect::booleanList(
				$selected,
				$name,
				$attribs
		);
	}
}
