<?php
/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * SobiproHelper
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class SobiproHelper
{
	const JS_LIB = 'components/com_sobipro/lib/js/';

	const JS_SOBIPRO = 'sobipro';

	const JS_JQNC = 'jqnc';

	const JS_JQUERY_MIGRATE = 'jquery-migrate';

	const JS_JQUERY_UI = 'jquery-ui';

	const JS_TIMEPICKER = 'timepicker';

	const JS_LANG_CALENDAR = '_calendar';

	const JS_CALENDAR_SEARCH = 'calendar_search';

	/**
	 * init
	 *
	 * @return	void
	 */
	public static function init()
	{
		$sp_config = &SPFactory::config();

		/* Load basic configuration settings */
		$sp_config->addIniFile('etc.config', true);
		$sp_config->addTable('spdb_config', 0);

		/* Initialise interface config setting */
		$sp_mainframe = SPFactory::mainframe();
		$sp_mainframe->getBasicCfg();

		/* initialise config */
		$sp_config->init();
	}

	/**
	 * loadAndExtend
	 *
	 * @param   int   $id              Param
	 * @param   bool  $init_structure  Param
	 *
	 * @return	object
	 */
	public static function loadAndExtend($id, $init_structure = false)
	{
		$entry = SPFactory::object($id);

		if (empty($entry))
		{
			return null;
		}

		// Setup section to load fields
		if ($init_structure)
		{
			$section_id = self::getSectionEntry($id);
			SPFactory::registry()->set('current_section', $section_id);
		}

		$class = SPLoader::loadModel($entry->oType, false, false);
		$entryModel = new $class;
		$entryModel->extend($entry);

		return $entryModel;
	}

	/**
	 * extendFields
	 *
	 * @param   array  &$fields  Param
	 *
	 * @return	void
	 */
	public static function extendFields(&$fields)
	{
		if (count($fields) > 0)
		{
			$fmod = SPLoader::loadModel('field');

			foreach ($fields as $i => $f)
			{
				$field = new $fmod;
				$field->extend($f);
				$fields[$i] = $field;
			}
		}
	}

	/**
	 * loadField
	 *
	 * @param   int  $id  Param
	 *
	 * @return	object
	 */
	public static function loadField($id)
	{
		$fmod = SPLoader::loadModel('field');
		$field = new $fmod;
		$field->init($id);

		return $field;
	}

	/**
	 * getVersion
	 *
	 * @return	void
	 */
	public static function getVersion()
	{
		$ext = FOFModel::getTmpInstance('Extensions', 'ExtlyModel');
		$ext->setState('element', 'com_sobipro');

		return $ext->getFirstItem();
	}

	/**
	 * isSobiPro11
	 *
	 * @return	void
	 */
	public static function isSobiPro11()
	{
		$comp = self::getVersion();

		if (!$comp)
		{
			return false;
		}

		$manifest = json_decode($comp->manifest_cache);

		return version_compare($manifest->version, '1.1', 'ge');
	}

	/**
	 * installAppScript
	 *
	 * @return	void
	 */
	public static function installAppScript()
	{
		$file = EHtml::getRelativeFile('js', 'com_xtdir/installapp.min.js');

		if ($file)
		{
			$dependencies = array();

			if (EXTLY_J25)
			{
				$dependencies['ajaxbutton'] = array('jquery1102');
			}

			$dependencies['installapp'] = array('extlycore');

			$paths = array(
				'ajaxbutton' => 'media/lib_extly/js/utils/jquery.xtajaxbutton.min'
			);

			Extly::initApp(CXTDIR_VERSION, $file, $dependencies, $paths);
		}
	}

	/**
	 * installApp
	 *
	 * @param   string  $source  Params
	 *
	 * @return	int.
	 *
	 * @since	1.0
	 */
	public static function installApp($source)
	{
		$filename = basename($source);

		@ob_end_clean();
		header('Content-type: text/plain');

		$_REQUEST['option'] = 'com_sobipro';
		$_REQUEST['task'] = 'extensions.install';
		$_REQUEST['sid'] = 0;
		$_REQUEST[SPFactory::mainframe()->token()] = 1;
		$_REQUEST['format'] = 'raw';
		unset($_REQUEST['view']);
		unset($_REQUEST['Itemid']);

		$_POST['ident'] = 'SobiProExtension-file';

		$src = JPATH_ROOT . $source;
		$dst = SOBI_PATH . '/tmp/' . $filename;
		JFile::copy($src, $dst);

		$file = array();
		$file['name'] = $filename;
		$file['type'] = 'application/zip';
		$file['tmp_name'] = $dst;
		$file['error'] = 0;
		$file['size'] = filesize($file['tmp_name']);

		$_FILES['SobiProExtension-file'] = $file;

		$class = SPLoader::loadController('adm.sobipro');
		$sobi = new $class(SPRequest::task());
		@$sobi->execute();
	}

	/**
	 * entryToggleEnabled
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return	int.
	 *
	 * @since	1.0
	 */
	public static function entryToggleEnabled($entry_id)
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$_REQUEST['option'] = 'com_sobipro';
		$_REQUEST['task'] = 'entry.toggle.enabled';
		$_REQUEST['sid'] = $entry_id;
		$_REQUEST[SPFactory::mainframe()->token()] = 1;
		$_REQUEST['format'] = 'raw';
		unset($_REQUEST['view']);
		unset($_REQUEST['Itemid']);

		$class = SPLoader::loadController('adm.sobipro');
		$sobi = new $class(SPRequest::task());

		self::_doNotRedirect();
		$sobi->execute();
	}

	/**
	 * entryToggleEnabled
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return	int.
	 *
	 * @since	1.0
	 */
	public static function entryToggleApproval($entry_id)
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$_REQUEST['option'] = 'com_sobipro';
		$_REQUEST['task'] = 'entry.toggle.approval';
		$_REQUEST['sid'] = $entry_id;
		$_REQUEST[SPFactory::mainframe()->token()] = 1;
		$_REQUEST['format'] = 'raw';
		unset($_REQUEST['view']);
		unset($_REQUEST['Itemid']);

		$class = SPLoader::loadController('adm.sobipro');
		$sobi = new $class(SPRequest::task());

		self::_doNotRedirect();
		$sobi->execute();
	}

	/**
	 * editUrl
	 *
	 * @param   string  $oType       Params
	 * @param   int     $object_id   Params
	 * @param   int     $section_id  Params
	 *
	 * @return	string.
	 *
	 * @since	1.0
	 */
	public static function editUrl($oType, $object_id, $section_id = null)
	{
		switch ($oType)
		{
			case 'entry':
				$url = 'index.php?option=com_sobipro&task=entry.edit&sid=' . $object_id . '&pid=' . $section_id;
				break;
			case 'category':
				$url = 'index.php?option=com_sobipro&task=category.edit&sid=' . $object_id;
				break;
			default:
				$url = 'index.php?option=com_sobipro&sid=' . $object_id;
		}

		$url = htmlspecialchars($url);

		return $url;
	}

	/**
	 * getUrl
	 *
	 * @param   string  $oType         Params
	 * @param   int     $object_sid    Params
	 * @param   int     $category_pid  Only for entries
	 * @param   int     $entry_nid     Only for entries
	 * @param   int     $itemid        Params
	 *
	 * @return	string.
	 *
	 * @since	1.0
	 */
	public static function getUrl($oType, $object_sid, $category_pid = null, $entry_nid = null, $itemid = null)
	{
		$url = null;

		$sef = (!FOFPlatform::getInstance()->isBackend());
		$forceItemId = false;

		switch ($oType)
		{
			case 'entry':
				$params = array(
								'title' => $entry_nid,
								'pid' => $category_pid,
								'sid' => $object_sid
				);

				if ($itemid)
				{
					$forceItemId = true;
					$params['itemid'] = $itemid;
				}

				$url = Sobi::Url($params, false, $sef, false, $forceItemId);
				break;
			case 'category':

				$url = Sobi::Url(
					array(
								'sid' => $object_sid
					),
					false,
					$sef,
					false,
					$forceItemId
				);
				break;
			default:
				$url = Sobi::Url(
					array(
								'sid' => $object_sid
					),
					false,
					$sef,
					false,
					$forceItemId
				);
		}

		if ($sef)
		{
			return $url;
		}
		else
		{
			return JUri::root() . $url;
		}
	}

	/**
	 * _doNotRedirect
	 *
	 * @return	void.
	 *
	 * @since	1.0
	 */
	private static function _doNotRedirect()
	{
		// HTTP/1.1
		header("Cache-Control: no-cache, must-revalidate");

		// Date in the past
		header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");

		// Just to avoid any further operation
		flush();
	}

	/**
	 * getCategoriesDesc
	 *
	 * @param   array  &$categories  Params
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function getCategoriesDesc(&$categories)
	{
		$desc = array();

		foreach ($categories as $key => $category)
		{
			$desc[] = $category['name'];
		}

		return implode(', ', $desc);
	}

	/**
	 * spAppInstallControl
	 *
	 * @param   string  $appPid       Param
	 * @param   string  $installUrl   Param
	 * @param   string  $downloadUrl  Param
	 * @param   string  $fieldName    Param
	 * @param   string  $idTag        Param
	 * @param   string  $class        Param
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function spAppInstallControl($appPid, $installUrl, $downloadUrl, $fieldName = 'installapp', $idTag = 'installapp', $class = 'install-app')
	{
		$installUrl = JUri::base() . $installUrl;
		$downloadUrl = JUri::root() . $downloadUrl;

		$pluginsModel = FOFModel::getTmpInstance('Plugins', 'XtdirModel');
		$pluginsModel->setState('pid', $appPid);

		$plugins = $pluginsModel->getItemList();
		$installed = (count($plugins) > 0);

		if ($installed)
		{
			$configurationUrl = 'index.php?option=com_sobipro';

			// Go for the update
			if (self::isSobiPro11())
			{
				echo EHtml::ajaxButtonControl($installUrl, 'COM_XTDIR_APP_INSTALLED_STATUS', $fieldName, 'COM_XTDIR_APP_INSTALLED_LABEL', 'COM_XTDIR_APP_INSTALLED_DESC', 'COM_XTDIR_APP_UPDATE_BUTTON', $idTag, $class, $configurationUrl);
			}
			else
			{
				echo EHtml::downloadButtonControl($downloadUrl, 'COM_XTDIR_APP_INSTALLED_STATUS', $fieldName, 'COM_XTDIR_APP_INSTALLED_LABEL', 'COM_XTDIR_APP_INSTALLED_DESC', 'COM_XTDIR_APP_DOWNLOAD_BUTTON', $idTag, 'btn-warning', $configurationUrl);
			}
		}
		else
		{
			// Go for the installation
			if (self::isSobiPro11())
			{
				echo EHtml::ajaxButtonControl($installUrl, 'COM_XTDIR_APP_NOT_INSTALLED_STATUS', $fieldName, 'COM_XTDIR_APP_NOT_INSTALLED_LABEL', 'COM_XTDIR_APP_NOT_INSTALLED_DESC', 'COM_XTDIR_APP_INSTALL_BUTTON', $idTag, $class);
			}
			else
			{
				echo EHtml::downloadButtonControl($downloadUrl, 'COM_XTDIR_APP_NOT_INSTALLED_STATUS', $fieldName, 'COM_XTDIR_APP_NOT_INSTALLED_LABEL', 'COM_XTDIR_APP_NOT_INSTALLED_DESC', 'COM_XTDIR_APP_DOWNLOAD_BUTTON', $idTag, 'btn-warning');
			}
		}
	}

	/**
	 * shownEntryName
	 *
	 * @param   object  &$entry  Params
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function shownEntryName(&$entry)
	{
		$buffer = array();
		$truncate_ents = EParameter::getComponentParam(CXTDIR, 'truncate_ents', UtilHelper::ENTS_TEXT_LENGHT);

		// Entry
		$txt = $entry->get('name');

		if ($truncate_ents)
		{
			$txt = UtilHelper::truncString($txt, $truncate_ents, true);
		}

		$buffer[] = htmlentities($txt, ENT_COMPAT, 'UTF-8');

		// Entry Edit Url
		$url = self::editUrl('entry', $entry->get('id'), $entry->get('primary'));
		$buffer[] = '<span class="entry-links pull-right"><a href="' . $url . '" target="_blank" data-original-title="' . JText::_('COM_XTDIR_SOBIPRO_EDITION_TOOLTIP') . '" rel="tooltip"><i class="xticon xticon-pencil"></i></a> ';

		// Entry Site Url
		$url = self::getUrl('entry', $entry->get('id'), $entry->get('primary'), Sobi::Cfg('sef.alias', true) ? $entry->get('nid') : $entry->get('name'));
		$buffer[] = '<a href="' . $url . '" target="_blank" data-original-title="' . JText::_('COM_XTDIR_PUBLIC_ENTRY_TOOLTIP') . '" rel="tooltip"><i class="xticon xticon-globe"></i></a></span>';

		return implode('', $buffer);
	}

	/**
	 * getSectionEntry
	 *
	 * @param   int  $entry_id  Params
	 *
	 * @return	int
	 */
	public static function getSectionEntry($entry_id)
	{
		$_db = JFactory::getDbo();
		$query = 'SELECT section FROM #__sobipro_field_data WHERE sid = ' . $_db->quote($entry_id) . " LIMIT 1";
		$_db->setQuery($query);

		return $_db->loadResult();
	}

	/**
	 * addTimepicker
	 *
	 * @param   string  $nid     Params
	 * @param   object  &$field  Params
	 *
	 * @return	void
	 */
	public static function addTimepicker($nid, &$field)
	{
		static $initialized = false;

		if (!$initialized)
		{
			$initialized = true;
			self::_initTimepicker();
		}

		$opt = array(
						'ampm' => (int) $field->get('inputFormClock12'),
						'dateFormat' => $field->get('inputForm'),
						'numberOfMonths' => (int) $field->get('numMonths'),
						'hourMin' => 0,
						'hourMax' => 24,
		);

		$timeOpt = $field->get('timeOpt');

		if (count($timeOpt))
		{
			$opt[ 'timeFormat' ] = implode(':', $timeOpt);
		}

		$opt = json_encode($opt);
		$sf = 0;
		$st = 0;
		$jsCode = "spCalendar( '{$nid}', {$opt}, {$sf}, {$st} );";

		return $jsCode;
	}

	/**
	 * initTimepicker
	 *
	 * @return	void
	 */
	private static function _initTimepicker()
	{
		// Style sheet
		$document = JFactory::getDocument();
		$document->addStyleSheet(JUri::root() . 'media/sobipro/css/timepicker.css');
		$document->addStyleSheet(JUri::root() . 'media/sobipro/css/jquery-ui/smoothness/smoothness.css');

		// Paths
		$paths = array();
		$paths[self::JS_SOBIPRO] = self::JS_LIB . self::JS_SOBIPRO;
		$paths[self::JS_JQNC] = self::JS_LIB . self::JS_JQNC;

		if (EXTLY_J25)
		{
			$paths[self::JS_JQUERY_MIGRATE] = self::JS_LIB . self::JS_JQUERY_MIGRATE;
		}

		$paths[self::JS_JQUERY_UI] = self::JS_LIB . self::JS_JQUERY_UI;
		$paths[self::JS_TIMEPICKER] = self::JS_LIB . self::JS_TIMEPICKER;

		$lang_calendar = Sobi::Lang(false) . self::JS_LANG_CALENDAR;
		$paths[$lang_calendar] = self::JS_LIB . $lang_calendar;

		$paths[self::JS_CALENDAR_SEARCH] = self::JS_LIB . self::JS_CALENDAR_SEARCH;

		Extly::addPath($paths);

		// Dependencies
		$dependencies = array();

		if (EXTLY_J25)
		{
			$dependencies[self::JS_SOBIPRO] = array('jquery1102');
		}

		$dependencies[self::JS_JQNC] = array(self::JS_SOBIPRO);

		if (EXTLY_J25)
		{
			$dependencies[self::JS_JQUERY_MIGRATE] = array(self::JS_JQNC);
			$dependencies[self::JS_JQUERY_UI] = array(self::JS_JQUERY_MIGRATE);
		}
		else
		{
			$dependencies[self::JS_JQUERY_UI] = array(self::JS_JQNC);
		}

		$dependencies[self::JS_TIMEPICKER] = array(self::JS_JQUERY_UI);
		$dependencies[$lang_calendar] = array(self::JS_TIMEPICKER);
		$dependencies[self::JS_CALENDAR_SEARCH] = array($lang_calendar);

		// Dependencies
		Extly::addDependency($dependencies);
	}

	/**
	 * getPlugin
	 *
	 * @param   string  $plugin  Param
	 *
	 * @return	object
	*/
	public static function getPlugin($plugin)
	{
		if (SPLoader::translatePath('opt.plugins.' . $plugin . '.init'))
		{
			$pc = SPLoader::loadClass($plugin . '.init', false, 'plugin');

			return new $pc($plugin);
		}
	}

	/**
	 * generateMjRadius
	 *
	 * @param   bool  $mj_rs_js  Param
	 *
	 * @return	string
	*/
	public static function generateMjRadius($mj_rs_js)
	{
		$plugin = 'mjradius';
		$mjradius = self::getPlugin($plugin);
		$view = array();
		$mjradius->SearchDisplay($view);

		if ($mj_rs_js)
		{
			$js = SPFactory::header()->getData('js');
			$document = JFactory::getDocument();
			$document->addScriptDeclaration(implode("\n", $js));
		}

		return $view[$plugin];
	}
}
