<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.error.error');

/**
 * StatsHelper Helper class.
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
 */
class StatsHelper
{
	/**
	 * cleanTotals
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public function cleanTotals()
	{
		$_db = JFactory::getDbo();
		$query = "TRUNCATE `#__xtdir_totals`;";
		$_db->setQuery($query);
		$_db->execute();
	}

	/**
	 * calculateTotals
	 *
	 * @param   mixed  $section  the params
	 *
	 * @return  array
	 */
	public function calculateTotals($section)
	{
		include_once JPATH_XTDIR . '/api/sobipro_api.php';

		$_db = JFactory::getDbo();
		$query = $_db->getQuery(true);

		$query->select('path');
		$query->from($_db->quoteName('#__xtdir_trees') . ' AS t');
		$query->from($_db->quoteName('#__sobipro_object') . ' AS e');
		$query->where('t.id = e.id');
		$query->where('oType = ' . $_db->quote('entry'));
		$query->where('section = ' . $_db->quote($section));

		$logger = XtdirLogger::getInstance();

		$time = microtime(true);
		$logger->log(JLog::INFO, 'Stats start');

		$logger->log(JLog::INFO, 'Stats: ' . $query);

		$_db->setQuery($query);

		$totals = array();
		$paths = $_db->loadColumn();

		if ((!$paths) || (count($paths) == 0))
		{
			$logger->log(JLog::WARNING, 'Empty Directory or No selected section.');
			$logger->log(JLog::WARNING, 'Paths', $paths);
		}
		else
		{
			foreach ($paths as $path)
			{
				$logger->log(JLog::INFO, 'Entry: ' . $path);

				$categories = explode('-', $path);

				foreach ($categories as $subcateg)
				{
					if (array_key_exists($subcateg, $totals))
					{
						$totals[$subcateg] = $totals[$subcateg] + 1;
					}
					else
					{
						$totals[$subcateg] = 1;
					}
				}
			}
		}

		foreach ($totals as $key => $value)
		{
			if ($key)
			{
				$c = SPFactory::object($key);
				$name = $_db->q($c->name);

				$query = "INSERT INTO `#__xtdir_totals` (`section_id`, `category_id`, `category_name`, `total`) VALUES({$section}, {$key}, {$name}, {$value});";
				$_db->setQuery($query);

				try
				{
					$_db->execute();

					if ($error = $_db->getErrorMsg())
					{
						$logger->log(JLog::ERROR, $error);
					}
				}
				catch (Exception $e)
				{
					$logger->log(JLog::ERROR, $e->getMessage());
				}
			}
		}

		$logger->log(JLog::INFO, 'Stats End - ' . round(microtime(true) - $time, 3) . ' seconds');

		return $totals;
	}

	/**
	 * getTotals
	 *
	 * @param   mixed  $section  the params
	 *
	 * @return  int
	 */
	public function getTotals($section)
	{
		$_db = JFactory::getDbo();
		$query = $_db->getQuery(true);

		$query->select('category_id, total');
		$query->from($_db->quoteName('#__xtdir_totals'));
		$query->where('section_id = ' . $_db->quote($section));

		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'Stats query: ' . $query);

		$_db->setQuery($query);

		$totals = $_db->loadAssocList('category_id', 'total');

		return $totals;
	}

	/**
	 * getEntriesCatsStats
	 *
	 * @param   int  $section  the params
	 *
	 * @return  int
	 */
	public static function getEntriesCatsStats($section)
	{
		$_db = JFactory::getDbo();
		$query = $_db->getQuery(true);

		$query->select('total');
		$query->from($_db->quoteName('#__xtdir_totals'));
		$query->where('category_id = ' . $_db->quote($section));
		$_db->setQuery($query);
		$total = $_db->loadResult();

		return $total;
	}

	/**
	 * getEntriesStats
	 *
	 * @param   int  $section  the params
	 *
	 * @return  int
	 */
	public static function getEntriesStats($section)
	{
		$objectsModel = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$objectsModel->setState('section', $section);
		$objectsModel->setState('oType', 'entry');
		$objectsModel->setState('state', 1);
		$objectsModel->setState('approved', 1);

		$total = $objectsModel->getTotal();

		return $total;
	}
}
