<?php
/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * UtilHelper
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
*/
class UtilHelper
{
	const ENTS_TEXT_LENGHT = 35;
	const CATS_TEXT_LENGHT = 35;
	const DESCS_TEXT_LENGHT = 160;

	/**
	 * cleanListOfNumerics
	 *
	 * @param   mixed  $listOfNumerics  the params
	 *
	 * @return  the list
	 *
	 * @since   1.0
	 */
	public static function cleanListOfNumerics($listOfNumerics)
	{
		return preg_replace('/[^,0-9]/', '', $listOfNumerics);
	}

	/**
	 * getSearchImage
	 *
	 * @param   mixed  $button_text  the params
	 *
	 * @return  the list
	 *
	 * @since   1.0
	 */
	public static function getSearchImage($button_text)
	{
		$img = JHtml::_('image', 'searchButton.gif', $button_text, null, true, true);

		return $img;
	}

	/**
	 * loadSection
	 *
	 * @param   object  &$view       Param
	 * @param   object  $section_id  Param
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public static function loadSection(&$view, $section_id = null)
	{
		if (!$section_id)
		{
			$section_id = JFactory::getApplication()->input->get('section_id');
		}

		$model = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$model->setState('id', $section_id);
		$model->setState('oType', 'section');
		$sections = $model->getItemList();

		if (count($sections))
		{
			$section = $sections[0];
			$section_id = $section->id;
			$view->assignRef('section_id', $section->id);
			$view->assignRef('section_name', $section->name);
		}

		$view->getModel()->setState('section_id', $section_id);
		$view->assignRef('section_id', $section_id);
	}

	/**
	 * loadCategory
	 *
	 * @param   object  &$view        Param
	 * @param   int     $section_id   Param
	 * @param   int     $category_id  Param
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public static function loadCategory(&$view, $section_id, $category_id = null)
	{
		if (!$category_id)
		{
			$category_id = JFactory::getApplication()->input->get('category_id');
		}

		$model = FOFModel::getTmpInstance('Objects', 'XtdirModel');
		$model->setState('id', $category_id);
		$model->setState('section_id', $section_id);
		$categories = $model->getItemList();

		if (count($categories))
		{
			$category = $categories[0];
			$view->assignRef('category_id', $category->id);
			$view->assignRef('category_name', $category->name);
		}

		$view->getModel()->setState('category_id', $category_id);
	}

	/**
	 * truncString.
	 *
	 * @param   string   $text       Param
	 * @param   integer  $max_chars  Param
	 * @param   bool     $withDots   Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	public static function truncString($text, $max_chars, $withDots = false)
	{
		$length = JString::strlen($text);

		if ($length > $max_chars)
		{
			if ($withDots)
			{
				// -Dots
				// -2 Utf8 case
				$max_chars = $max_chars - 3 - 2;
			}

			$text = JString::substr($text, 0, $max_chars);

			// Yes, it can return more characters, but utf8_strlen($text) is Ok, so ...
			$l = $max_chars;

			while (strlen($text) > $max_chars)
			{
				$l--;
				$text = JString::substr($text, 0, $l);
			}

			if ($withDots)
			{
				$text = $text . '...';
			}
		}

		return $text;
	}

	/**
	 * getIconView.
	 *
	 * @param   string  $view  Param
	 *
	 * @return	string
	 */
	public static function getIconView($view)
	{
		switch ($view)
		{
			case 'cpanel':
			case 'cpanels':
				$icon = 'xticon xticon-dashboard';
				break;
			case 'object':
			case 'objects':
				$icon = 'xticon xticon-arrow-circle-up';
				break;
			case 'promoentriescfg':
			case 'promoentriescfgs':
				$icon = 'xticon xticon-cog';
				break;
			case 'promotions':
			case 'promotion':
				$icon = 'xticon xticon-th-list';
				break;
			case 'promoorders':
			case 'promoorder':
				$icon = 'xticon xticon-money';
				break;
			case 'pagescfg':
			case 'pagescfgs':
			case 'pagesescfgs':
				$icon = 'xticon xticon-briefcase';
				break;
			case 'system':
			case 'systems':
				$icon = 'xticon xticon-bolt';
				break;
			case 'search':
			case 'searchs':
				$icon = 'xticon xticon-search';
				break;
			case 'finder':
			case 'finders':
				$icon = 'xticon xticon-filter';
				break;
			case 'trees':
			case 'trees':
				$icon = 'xticon xticon-sitemap';
				break;
			case 'ipncfg':
			case 'ipncfgs':
				$icon = 'xticon xticon-anchor';
				break;
			default:
				$icon = 'xticon xticon-times-circle';
				break;
		}

		return $icon;
	}
}
