<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirModelFields
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class XtdirModelFields extends FOFModel
{
	/**
	 * buildQuery
	 *
	 * @param   bool  $overrideLimits  Param
	 *
	 * @return	FOFQuery
	 */
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();

		$query = $db->getQuery(true)->select('f.*')->from($db->quoteName('#__sobipro_field', 'f'));

		$fltEnabled = $this->getState('enabled', null, 'int');

		if ($fltEnabled != '')
		{
			$query->where($db->qn('enabled') . ' = ' . $db->q($fltEnabled));
		}

		$fltInSearch = $this->getState('inSearch', null, 'int');

		if ($fltInSearch != '')
		{
			$query->where($db->qn('inSearch') . ' = ' . $db->q($fltInSearch));
		}

		$fltSection = $this->getState('section', null, 'int');

		if ($fltSection != '')
		{
			$query->where('f.' . $db->qn('section') . ' = ' . $db->q($fltSection));
		}

		$fltSections = $this->getState('sections', null, 'string');

		if ($fltSections != '')
		{
			$query->where('f.' . $db->qn('section') . ' IN (' . $fltSections . ')');
		}

		$fltIds = $this->getState('ids', null, 'string');

		if ($fltIds != '')
		{
			$query->where($db->qn('fid') . ' IN (' . $fltIds . ')');
		}

		$fltType = $this->getState('fieldType', null, 'cmd');

		if ($fltType)
		{
			$query->where($db->qn('fieldType') . ' = ' . $db->q($fltType));
		}

		/*
		 * inbox can be search filter select or range textarea can also be a search field $notextsearch = $this->getState('notextsearch', 0, 'int'); if ($notextsearch) { $query->where($db->qn('fieldType') . ' NOT IN (' . $db->q('inbox') . ',' . $db->q('textarea') . ')'); }
		 */

		$nocategory = $this->getState('nocategory', 0, 'int');

		if ($nocategory)
		{
			$query->where($db->qn('fieldType') . ' NOT IN (' . $db->q('category') . ')');
		}

		$isNameField = $this->getState('isNameField', 0, 'int');

		if ($isNameField)
		{
			$query->from($db->quoteName('#__sobipro_config', 'cfg'));

			$query->where('cfg.section = f.section');
			$query->where('f.fid = cfg.sValue');
			$query->where('cfg.sKey = ' . $db->Quote('name_field'));
		}

		$order = $this->getState('filter_order', 'fid', 'string');
		$dir = $this->getState('filter_order_Dir', 'ASC', 'cmd');
		$query->order($order . ' ' . $dir);

		return $query;
	}

	/**
	 * getSearchFields
	 *
	 * @param   int  $section       Param
	 * @param   int  $notextsearch  Param
	 * @param   int  $nocategory    Param
	 *
	 * @return	array
	 */
	public function getSearchFields($section = null, $notextsearch = false, $nocategory = false)
	{
		$this->setState('enabled', 1);
		$this->setState('inSearch', 1);
		$this->setState('section', $section);
		$this->setState('notextsearch', $notextsearch);
		$this->setState('nocategory', $nocategory);

		$this->setState('filter_order', 'section,position');

		return $this->getItemList(true);
	}

	/**
	 * getCategorySearchField
	 *
	 * @param   int  $section  Param
	 *
	 * @return	array
	 */
	public function getCategorySearchField($section)
	{
		$this->setState('enabled', 1);
		$this->setState('inSearch', 1);
		$this->setState('section', $section);
		$this->setState('fieldType', 'category');

		$fields = $this->getItemList(true);
		SobiproHelper::extendFields($fields);

		return $fields;
	}

	/**
	 * getSPSearchFields
	 *
	 * @param   int  $section       Param
	 * @param   int  $notextsearch  Param
	 * @param   int  $nocategory    Param
	 *
	 * @return	array
	 */
	public function getSPSearchFields($section = null, $notextsearch = false, $nocategory = false)
	{
		$fields = $this->getSearchFields($section, true);
		SobiproHelper::extendFields($fields);

		$results = array();

		foreach ($fields as $field)
		{
			$type = $field->get('fieldType');

			switch ($type)
			{
				case 'inbox':
					$params = $field->get('params');

					if ((is_array($params)) && (array_key_exists('searchMethod', $params)))
					{
						$searchMethod = $params['searchMethod'];

						if ($searchMethod != 'general')
						{
							$results[] = $field;
						}
					}
					else
					{
						$results[] = $field;
					}
					break;
					break;
				case 'textarea':
					break;
				default:
					$results[] = $field;
			}
		}

		return $results;
	}

	/**
	 * getSearchFieldsIds
	 *
	 * @return	array
	 */
	public function getSearchFieldsIds()
	{
		$fields = $this->getSearchFields();

		$search_field_list = null;

		if (count($fields) > 0)
		{
			$search_field_list = $this->getFids($fields);
		}

		return $search_field_list;
	}

	/**
	 * getFids
	 *
	 * @param   array  &$fields  Param
	 *
	 * @return	array
	 */
	public static function getFids(&$fields)
	{
		return array_map(create_function('$f', 'return $f->fid;'), $fields);
	}
}
