<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirModelObjects
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class XtdirModelObjects extends FOFModel
{
	/**
	 * buildQuery
	 *
	 * @param   bool  $overrideLimits  Param
	 *
	 * @return	FOFQuery
	 */
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();

		$query = $db->getQuery(true)->from($db->quoteName('#__sobipro_object') . ' AS o');

		$fltType = $this->getState('oType', null, 'cmd');

		if ($fltType == 'entry')
		{
			$query->select('o.*, pe.id as promoted_id, pe.promotype_id as promotype_id, IF(ISNULL(pe.ordering), ' . PromotedHelper::ENTRY_NOT_PROMOTED . ', pe.ordering) fordering, pe.promoentry_id as promoentry_id, pe.tiertype_id as tiertype_id, pe.promoorder_id as promoorder_id');
		}
		else
		{
			$query->select('o.*');
		}

		$fltName = $this->getState('name', null, 'string');

		if ($fltName)
		{
			$fltName = "%$fltName%";
			$query->where('(o.' . $db->qn('name') . ' LIKE ' . $db->q($fltName) . ' OR ' . 'o.' . $db->qn('nid') . ' LIKE ' . $db->q($fltName) . ')');
		}

		$fltId = $this->getState('id', null, 'int');

		if (!empty($fltId))
		{
			$query->where('o.' . $db->qn('id') . ' = ' . $db->q($fltId));
		}

		$fltState = $this->getState('state', null, 'int');

		if (($fltState === 1) || ($fltState === 0))
		{
			$query->where('o.' . $db->qn('state') . ' = ' . $db->q($fltState));
		}

		$fltApproved = $this->getState('approved', null, 'int');

		if (($fltApproved === 1) || ($fltApproved === 0))
		{
			$query->where('o.' . $db->qn('approved') . ' = ' . $db->q($fltApproved));
		}

		$fltValid = $this->getState('valid', 0, 'int');

		if ($fltValid)
		{
			$now = JFactory::getDate();
			$nowSql = $db->q($now->toSql());

			$query->where('o.' . $db->qn('validSince') . ' <= ' . $nowSql);
			$query->where('(' . 'o.' . $db->qn('validUntil') . ' >= ' . $nowSql . ' OR ' . 'o.' . $db->qn('validUntil') . ' = 0' . ')');
		}

		$fltPromotype = $this->getState('promotype_id', null, 'int');
		$fltPromotype = chr($fltPromotype);
		$valid = array(
						'M' => 'Entry',
						'A' => 'Automatic',
						'N' => 'None'
		);

		if (array_key_exists($fltPromotype, $valid))
		{
			switch ($fltPromotype)
			{
				case 'M':
					$query->where($db->qn('pe.promotype_id') . ' = ' . $db->q('E'));
					break;
				case 'A':
					$query->where($db->qn('pe.promotype_id') . " IN ('S', 'C')");
					break;
				case 'N':
					$query->where($db->qn('pe.promotype_id') . " NOT IN ('E', 'S', 'C')");
					break;
			}
		}

		$valid = array(
						'E' => 'Entry',
						'C' => 'Category',
						'S' => 'Section'
		);

		if (array_key_exists($fltPromotype, $valid))
		{
			$query->where($db->qn('pe.promotype_id') . ' = ' . $db->q($fltPromotype));
		}

		if ($fltType)
		{
			$query->where('o.' . $db->qn('oType') . ' = ' . $db->q($fltType));
		}

		$fltCategory = $this->getState('category_id', null, 'int');
		$fltCategory = ($fltCategory ? $fltCategory : $this->getState('category', null, 'int'));

		if ($fltCategory)
		{
			$query->join('LEFT', $db->quoteName('#__xtdir_final_categorypromoted') . ' AS pe ON o.id = pe.entry_id and pe.category_id = ' . $db->q($fltCategory));

			$subquery = $db->getQuery(true)->select('r.id')->from($db->quoteName('#__sobipro_relations') . ' AS r')->where('r.pid = ' . $db->q($fltCategory));
			$query->where('o.id IN (' . $subquery->__toString() . ')');
		}
		else
		{
			$query->join('LEFT', $db->quoteName('#__xtdir_final_sectionpromoted') . ' AS pe ON o.id = pe.entry_id');
		}

		$fltSections = $this->getState('sections', null, 'string');

		if (!empty($fltSections))
		{
			$theSections = explode(',', $fltSections);

			if (count($theSections) == 1)
			{
				$this->setState('sections', null);
				$this->setState('section_id', $fltSections);
			}
			else
			{
				$subquery = $db->getQuery(true)->select('t.id')->from($db->quoteName('#__xtdir_trees') . ' AS t')->where('t.id = o.id')->where('t.section in (' . $fltSections . ')');
				$query->where('o.id IN (' . $subquery->__toString() . ')');
			}
		}

		$fltSection = $this->getState('section_id', null, 'int');
		$fltSection = ($fltSection ? $fltSection : $this->getState('section', null, 'int'));

		if (($fltSection) && (!$fltCategory))
		{
			$subquery = $db->getQuery(true)->select('t.id')->from($db->quoteName('#__xtdir_trees') . ' AS t')->where('t.id = o.id')->where('t.section = ' . $db->q($fltSection));
			$query->where('o.id IN (' . $subquery->__toString() . ')');
		}

		$fltOwner = $this->getState('owner', null, 'int');

		if ($fltOwner)
		{
			$query->where('o.' . $db->qn('owner') . ' = ' . $db->q($fltOwner));
		}

		$order = $this->getState('filter_order', 'o.id', 'string');
		$dir = $this->getState('filter_order_Dir', 'ASC', 'cmd');
		$query->order($order . ' ' . $dir);

		/*
		$logger = XtdirLogger::getInstance();
		$logger->log(JLog::INFO, 'XtdirModelObjects: ' . $query->__toString());
		*/

		return $query;
	}
}
