<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * XtdirModelPromoOrders
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 * @since       1.0
*/
class XtdirModelPromoOrders extends FOFModel
{
	/**
	 * buildQuery
	 *
	 * @param   bool  $overrideLimits  Param
	 *
	 * @return	FOFQuery
	 */
	public function buildQuery($overrideLimits = false)
	{
		$db = $this->getDbo();

		$query = $db->getQuery(true)->select('po.*, u.id as user_id, u.username, e.name as entry_name')->from($db->quoteName('#__xtdir_promoorders', 'po'))->join('LEFT', '#__sobipro_object as e on po.entry_id = e.id')->join('LEFT', '#__users as u on e.owner = u.id');

		$fltId = $this->getState('id', null, 'int');

		if ($fltId != '')
		{
			$query->where($db->qn('po.id') . ' = ' . $db->q($fltId));
		}

		$fltUser = $this->getState('user', null, 'string');

		if ($fltUser != '')
		{
			$query->where('(po.' . $db->qn('user_id') . ' = ' . $db->q($fltUser) . ' OR ' . 'u.' . $db->qn('username') . ' LIKE ' . $db->q('%' . $fltUser . '%') . ' OR ' . 'u.' . $db->qn('email') . ' LIKE ' . $db->q('%' . $fltUser . '%') . ')');
		}

		$fltEntry = $this->getState('entry', null, 'string');

		if ($fltEntry != '')
		{
			$query->where('(po.' . $db->qn('entry_id') . ' = ' . $db->q($fltEntry) . ' OR ' . 'e.' . $db->qn('name') . ' LIKE ' . $db->q('%' . $fltEntry . '%') . ')');
		}

		$fltCategory = $this->getState('category', null, 'string');

		if ($fltCategory != '')
		{
			$subquery = $db->getQuery(true)->select('t.id')->from($db->quoteName('#__xtdir_tree', 't'))->from($db->quoteName('#__sobipro_object', 'c'))->where('t.id = po.entry_id')->where('t.parent = c.id')->where('(t.' . $db->qn('parent') . ' = ' . $db->q($fltCategory) . ' OR ' . 'c.' . $db->qn('name') . ' LIKE ' . $db->q('%' . $fltCategory . '%') . ')');

			$query->where('(po.entry_id = (' . $subquery . ')');
		}

		$fltOrderStatus = $this->getState('order_status', null, 'cmd');

		if ($fltOrderStatus != '')
		{
			$query->where($db->qn('po.order_status') . ' = ' . $db->q($fltOrderStatus));
		}

		$fltOrderPaymentSerialNumber = $this->getState('payment_serial_number', null, 'string');

		if ($fltOrderPaymentSerialNumber != '')
		{
			$query->where($db->qn('po.payment_serial_number') . ' = ' . $db->q($fltOrderPaymentSerialNumber));
		}

		$order = $this->getState('filter_order', 'po.id', 'cmd');

		if (!in_array($order, array_keys($this->getTable()->getData())))
		{
			$order = 'po.id';
		}

		$dir = $this->getState('filter_order_Dir', 'ASC', 'cmd');
		$query->order($order . ' ' . $dir);

		return $query;
	}

	/**
	 * Binds the data to the model and tries to save it
	 *
	 * @param   array|object  $data  The source data array or object
	 *
	 * @return  boolean  True on success
	 */
	public function save($data)
	{
		$result = parent::save($data);

		if (is_object($data))
		{
			PromotedHelper::index($data->entry_id);
		}
		else
		{
			PromotedHelper::index($data['entry_id']);
		}

		return $result;
	}
}
