<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
// defined('_JEXEC') or die('Restricted access');

// To avoid JUri warning
$_SERVER['HTTP_HOST'] = 'localhost';

define('EXTLY_CRONJOB_RUNNING', true);
define('XTDIR_CRONJOB_RUNNING', true);

/**
 * starts the Xtdir cronjob
 * Call this file form crontab.
 **/

// Make sure we're being called from the command line, not a web interface
if (array_key_exists('REQUEST_METHOD', $_SERVER))
{
	die();
}

/**
 * Xtdir CLI Bootstrap
 *
 * Run the framework bootstrap with a couple of mods based on the script's needs
*/

// We are a valid entry point.
define('_JEXEC', 1);
define('DS', DIRECTORY_SEPARATOR);

// Load system defines
if (file_exists(dirname(dirname(__FILE__)) . '/defines.php'))
{
	require_once dirname(dirname(__FILE__)) . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	// ..../cli
	$path = dirname(dirname(__FILE__));
	$dirs = explode(DIRECTORY_SEPARATOR, $path);

	$path = implode(DIRECTORY_SEPARATOR, $dirs);

	define('JPATH_BASE', $path);
	require_once JPATH_BASE . '/includes/defines.php';
}

// Get the framework.
require_once JPATH_LIBRARIES . '/import.php';

// Bootstrap the CMS libraries.
require_once JPATH_LIBRARIES . '/cms.php';

// Get the framework.
if (version_compare(JVERSION, '3.0.0', 'ge'))
{
	require_once JPATH_LIBRARIES . '/import.legacy.php';
}

// Force library to be in JError legacy mode
JError::$legacy = true;

// Import necessary classes not handled by the autoloaders
jimport('joomla.application.menu');
jimport('joomla.environment.uri');
jimport('joomla.event.dispatcher');
jimport('joomla.utilities.utility');
jimport('joomla.utilities.arrayhelper');
jimport('joomla.error.profiler');

// Import the configuration.
require_once JPATH_CONFIGURATION . '/configuration.php';

// System configuration.
$config = new JConfig;

// Configure error reporting to maximum for CLI output.
error_reporting(E_ALL);
ini_set('display_errors', 1);

if (!defined('XTDIR_API'))
{
	include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/xtdirapi.php';
}

// Load Library language
$lang = JFactory::getLanguage();

// Try the finder_cli file in the current language (without allowing the loading of the file in the default language)
// Fallback to the finder_cli file in the default language
$lang->load('xtdir_cli', JPATH_SITE, null, false, false) || $lang->load('xtdir_cli', JPATH_SITE, null, true);

JLoader::register('PlgSystemXtdir', JPATH_BASE . '/plugins/system/xtdir/xtdir.php');

/**
 * A command line cron job to run the Xtdir job.
 *
 * @package     Xtdir.CLI
 * @subpackage  com_xtdir
 * @since       2.5
*/
class XtdirCli extends JApplicationCli
{
	/**
	 * Start time for the posting process
	 *
	 * @var    string
	 * @since  2.5
	 */
	private $_time = null;

	/**
	 * Start time for each batch
	 *
	 * @var    string
	 * @since  2.5
	 */
	private $_qtime = null;

	/**
	 * Entry point for Xtdir CLI script
	 *
	 * @return  void
	 *
	 * @since   2.5
	 */
	public function doExecute()
	{
		// Print a blank line.
		$this->out(JText::_('XTDIR_CLI'));
		$this->out('============================');
		$this->out();

		$this->_index();

		// Print a blank line at the end.
		$this->out();
	}

	/**
	 * Run the job
	 *
	 * @return  void
	 *
	 * @since   2.5
	 */
	private function _index()
	{
		jimport('joomla.application.component.helper');

		// Disable caching.
		$config = JFactory::getConfig();
		$config->set('caching', 0);
		$config->set('cache_handler', 'file');

		// Starting Indexer.
		$this->out(JText::_('XTDIR_CLI_STARTING_PROCESS'), true);

		// Remove the script time limit.
		@set_time_limit(0);

		// Initialize the time value
		$this->_time = microtime(true);
		$profile = JProfiler::getInstance();

		JLoader::load('PlgSystemXtdir');

		$plugin = JPluginHelper::getPlugin('system', 'xtdir');
		$className = 'plgSystem' . $plugin->name;

		if (class_exists($className))
		{
			$now = date('r');
			$this->out($profile->mark('XTDir Indexer Job - Start'));

			$dispatcher = JDispatcher::getInstance();
			$plugin = new $className($dispatcher, (array) $plugin);

			if ($plugin)
			{
				$catalog_list = $plugin->params->def('catalog_list', null);
				$cronjob = true;

				$catalog_list_dump = 'All';

				if ((!empty($catalog_list)) && (count($catalog_list) > 0))
				{
					$catalog_list_dump = implode(',', $catalog_list);
				}

				echo "\nJob - Parameters catalog_list: {$catalog_list_dump}, cronjob: {$cronjob}\n";
				$plugin->execute(true);

				$now = date('r');
				$this->out($profile->mark('XTDir Indexer Job - End'));
			}
			else
			{
				echo 'ERROR: ' . $className . " not instantiated!";
			}
		}
		else
		{
			echo 'ERROR: ' . $className . " not found!";
		}

		// Total reporting.
		$this->out(JText::_('XTDIR_CLI_PROCESS_COMPLETE') . round(microtime(true) - $this->_time, 3) . " seconds");
	}
}

// Instantiate the application object, passing the class name to JCli::getInstance
// and use chaining to execute the application.
JApplicationCli::getInstance('XtdirCli')->execute();

JFactory::getSession()->close();
$_db = JFactory::getDbo();
$_db->close();
sleep(1);
