<?php

/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once JPATH_ADMINISTRATOR . '/components/com_xtdir/api/sobipro_api.php';
include_once JPATH_BASE . '/components/com_community/libraries/core.php';

/**
 * XtdirViewMyEntries
 *
 * @package     Extly.Components
 * @subpackage  com_xtdir
 * @since       1.0
 */
class XtdirViewMyentries extends FOFViewRaw
{
	/**
	 * Displays the view
	 *
	 * @param   string  $tpl  The template to use
	 *
	 * @return  boolean|null False if we can't render anything
	 */
	public function display($tpl = null)
	{
		$is_module = $this->input->get('is_module', false, 'bool');
		$this->set('is_module', $is_module);

		$user_mode = $this->input->get('user_mode', 0, 'int');
		$this->set('user_mode', $user_mode);

		$imagefield_id = $this->input->get('imagefield_id', null, 'array');
		$this->set('imagefield_id', $imagefield_id);

		$descfield_id = $this->input->get('descfield_id', null, 'array');
		$this->set('descfield_id', $descfield_id);

		$menu_itemid = $this->input->get('menu_itemid', null, 'int');
		$this->set('menu_itemid', $menu_itemid);

		$limit = $this->input->get('limit', 10, 'int');
		$this->set('limit', $limit);

		// Sections
		$catalog_list = $this->input->get('catalog_list', null, 'array');

		if ((empty($catalog_list) || (count($catalog_list) == 0)))
		{
			$catalog_list = SectionsHelper::findAll();
		}

		if ((empty($catalog_list) || (count($catalog_list) == 0)))
		{
			echo '<p>' . JText::_('COM_XTDIR_VIEW_MYENTRIES_NO_ENTRIES') . '</p>';

			return true;
		}

		$this->set('catalog_list', $catalog_list);

		// Just in case, it can't show multiple images or descs

		/*
		if (count($catalog_list) > 1)
		{
			$this->set('imagefield_id', 0);
			$this->set('descfield_id', 0);
		}
		*/

		// Logged User
		if ($user_mode == 0)
		{
			$user = JFactory::getUser();
		}
		// JomSocial Profile
		elseif ($user_mode == 1)
		{
			$userid = $this->input->get('userid', 0, 'int');
			$user = JFactory::getUser($userid);
		}
		// EasySocial Profile
		elseif ($user_mode == 2)
		{
			$user_id = $this->input->get('user_id', 0, 'int');

			if (empty($user_id))
			{
				$user_id = $this->input->get('id', 0, 'int');
			}

			$this->set('user_id', $user_id);
			$user = Foundry::user($user_id);
		}

		if (!$user->id)
		{
			echo '<p>' . JText::_('COM_XTDIR_VIEW_MYENTRIES_UNKNOWN_USER') . '(' . $user_mode . ')</p>';

			return true;
		}

		$this->set('user', $user);

		$model = $this->getModel();
		$model->reset();
		$model->setState('id', 0);

		// Not save the state to the session
		$model->savestate(0);

		$model->setState('state', 1);
		$model->setState('approved', 1);
		$model->setState('valid', 1);
		$model->setState('oType', 'entry');
		$model->setState('sections', implode(',', $catalog_list));
		$model->setState('owner', $user->id);
		$model->setState('limit', $limit);

		$filter_order = $this->input->get('filter_order', 'o.name', 'string');
		$model->setState('filter_order', $filter_order, 'string');

		return parent::display($tpl);
	}

	/**
	 * preProcess
	 *
	 * @param   array  $items  Params
	 *
	 * @return  array
	 */
	public function _preProcess($items)
	{
		$truncate_ents = EParameter::getComponentParam(CXTDIR, 'truncate_ents', UtilHelper::ENTS_TEXT_LENGHT);

		$results = array();

		foreach ($items as $item)
		{
			$entry = SobiProHelper::loadAndExtend($item->id);

			$categories = $entry->getCategories();
			$categoriesDesc = SobiProHelper::getCategoriesDesc($categories);

			$title = $entry->get('name');

			if ($truncate_ents)
			{
				$title = UtilHelper::truncString($title, $truncate_ents, true);
			}

			$results[$item->id] = array(
							"id" => $item->id,
							"title" => $title,
							"nid" => $entry->get('nid'),
							"publish_up" => $entry->get('publish_up'),
							"cats" => $categoriesDesc,
							"entry" => $entry
			);
		}

		return $results;
	}

	/**
	 * esWidgetHeader
	 *
	 * @return  string
	 */
	public function esWidgetHeader()
	{
		return '<div class="es-widget">
	<div class="es-widget-head">
		<div class="pull-left widget-title">' .
			JText::_('APP_ESSOBIPROMYENTRIES_PROFILE_WIDGET_TITLE') . '</div>
	</div>
	<div class="es-widget-body">';
	}

	/**
	 * esWidgetFooter
	 *
	 * @return  string
	 */
	public function esWidgetFooter()
	{
		return '</div>
</div>';
	}

	/**
	 * showImage
	 *
	 * @param   object  &$entry         Params
	 * @param   int     $imagefield_id  Params
	 *
	 * @return  void
	 */
	public function showImage(&$entry, $imagefield_id)
	{
		$field = SobiproHelper::loadField($imagefield_id);
		$nid = $field->get('nid');

		$nids = $entry->getFields();

		if (!array_key_exists($nid, $nids))
		{
			return;
		}

		$image = $entry->getField($nid);

		if (!empty($image))
		{
			$image = $image->struct();

			$icon = $image['_attributes'];
			$icon = $icon['icon'];
			echo '<img src="' . $icon . '" style="float:right;">';
		}
	}

	/**
	 * showDesc
	 *
	 * @param   object  &$entry          Params
	 * @param   int     $descfield_id    Params
	 * @param   int	    $truncate_descs  Params
	 *
	 * @return  void
	 */
	public function showDesc(&$entry, $descfield_id, $truncate_descs)
	{
		$field = SobiproHelper::loadField($descfield_id);
		$nid = $field->get('nid');

		$nids = $entry->getFields();

		if (!array_key_exists($nid, $nids))
		{
			return;
		}

		$desc = $entry->get($nid);

		if ($truncate_descs)
		{
			$desc = UtilHelper::truncString($desc, $truncate_descs, true);
		}

		echo '<small>' . $desc . '</small>';
	}
}
