/**
 * @package     Extly.Components
 * @subpackage  com_xtdir - Extended Directory for SobiPro
 *
 * @author      Prieco S.A.
 * @copyright   Copyright (C) 2007 - 2014 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

define('index', [ 'extlycore' ], function(Core) {
	"use strict";
	
	var IndexBegin = Core.ExtlyModel.extend({
		
		url : function() {
			return 'index.php?option=com_xtdir&view=systems&task=getIndexBegin';
		}

	});
	
	var IndexStatus = Core.ExtlyModel.extend({
		
		url : function() {
			return 'index.php?option=com_xtdir&view=systems&task=getIndexStatus';
		}

	});
	
	var IndexEnd = Core.ExtlyModel.extend({
		
		url : function() {
			return 'index.php?option=com_xtdir&view=systems&task=getIndexEnd';
		}

	});	

	var IndexView = Backbone.View.extend({

		events : {
			'click #toolbar-process button' : 'onIndexBegin',
			'click #toolbar-process .toolbar' : 'onIndexBegin',
			'click #cpanel-index' : 'onIndexBegin'
		},

		initialize : function() {			
			var button;
			
			button = this.$('#toolbar-process button');
			button.attr('onclick', '');			
			button.click(function(e) {
				e.preventDefault();
			});
			
			button = this.$('#toolbar-process .toolbar');
			button.attr('onclick', '');
			button.click(function(e) {
				e.preventDefault();
			});			
			
			button = this.$('#cpanel-index');
			button.attr('onclick', '');
			button.click(function(e) {
				e.preventDefault();
			});
		},

		onIndexBegin: function() {
			var indexStart = new IndexBegin(),
				token = this.$('#token').attr('name');
			
			this.$('.index-progress').removeClass('hide');
			this.$('.index-progress .success-message').addClass('hide');
			
			this.$('#toolbar-process button').addClass('disabled');
			this.$('#toolbar-process .toolbar').addClass('disabled');			
			this.$('#cpanel-index').addClass('disabled');

			indexStart.on('change', this.loadIndexBegin, this);
			indexStart.fetch({
				data : {
					_token : token
				},
				
				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});
		},
		
		loadIndexBegin: function(message) {
			var name;
			
			if (message.get('status')) {
				this.catalogs = message.get('catalogs');
				
				this.nextCatalog();
				this.onIndexStatus(0);
			} else {
				alert(message.get('error_message'));
			}		
		},		
		
		onIndexStatus: function(is_continue) {
			var indexStatus = new IndexStatus(),
				token = this.$('#token').attr('name');
			
			indexStatus.on('change', this.loadIndexStatus, this);
			indexStatus.fetch({
				data : {
					_token : token,
					sectionId : this.currentCatalog.id,
					isContinue : is_continue
				},
				
				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});			
		},
		
		loadIndexStatus: function(message) {
			var completed;
			
			if (message.get('status')) {
				completed = message.get('completed');

				this.assignTotal(message.get('total'));				
				if (completed) {
					this.resetBar(100);
					
					if (_.size(this.catalogs) === 0) {
						this.onIndexEnd();						
					} else {
						this.nextCatalog();	
						this.onIndexStatus(0);
					}					
				} else {
					this.incBar();
					this.onIndexStatus(1);
				}				
			} else {
				alert(message.get('error_message'));
			}		
		},
		
		onIndexEnd: function() {
			var indexEnd = new IndexEnd(),
				token = this.$('#token').attr('name');
			
			indexEnd.on('change', this.loadIndexEnd, this);
			indexEnd.fetch({
				data : {
					_token : token
				},
				
				wait : true,
				dataType : 'text',
				error : function(model, fail, xhr) {
					alert(fail.responseText);
				}
			});
		},
		
		loadIndexEnd: function(message) {
			if (message.get('status')) {
				this.$('.index-progress .success-message').removeClass('hide');
				
				this.$('#toolbar-process button').removeClass('disabled');
				this.$('#toolbar-process .toolbar').removeClass('disabled');				
				this.$('#cpanel-index').removeClass('disabled');
			} else {
				alert(message.get('error_message'));
			}		
		},
		
		assignSectionName: function(name) {
			var sectionName = this.$('.index-progress .section');
			sectionName.val(name);
		},
		
		assignTotal: function(total) {
			var sectionTotal = this.$('.index-progress .total');
			sectionTotal.val(total);			
		},
		
		resetBar: function(value) {
			var bar = this.$('.index-progress .bar');
			
			if (value) {
				this.barIndex = value;
			} else {
				this.barIndex = 1;
			}			
			bar.attr('style', 'width: ' + this.barIndex +'%;');
		},		

		incBar: function() {
			var bar = this.$('.index-progress .bar');
			
			this.barIndex += 5;
			if (this.barIndex > 100) {
				this.resetBar();
			} else {
				bar.attr('style', 'width: ' + this.barIndex + '%;');
			}
		},	
		
		nextCatalog: function() {
			var name;
			
			this.currentCatalog = _.first(this.catalogs); 
			this.catalogs = _.rest(this.catalogs);
			
			name = this.currentCatalog.name + ' (' + this.currentCatalog.id + ')';
			this.assignSectionName(name);
			this.assignTotal(0);
			this.resetBar();
		}

	}),

	indexView = new IndexView({
		el : jQuery('body')
	});

});
